@@ -40,7 +40,7 @@
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -155,19 +155,32 @@ Neck_TurnToRelative::execute( PlayerAgent * agent )
 bool
 Neck_TurnToPoint::execute( PlayerAgent * agent )
 {
-    AngleDeg target_rel_angle
-        = agent->effector().queuedNextAngleFromBody( M_point );
+    const Vector2D next_pos = agent->effector().queuedNextMyPos();
+    const AngleDeg next_body = agent->effector().queuedNextMyBody();
+    const double next_view_width = agent->effector().queuedNextViewWidth().width() * 0.5;
 
-    //if ( ( agent->world().self().neck() - target_rel_angle ).abs() < 1.0 )
-    //{
-    //    return false;
-    //}
+    for ( std::vector< Vector2D >::const_iterator p = M_points.begin();
+          p != M_points.end();
+          ++p )
+    {
+        Vector2D rel_pos = *p - next_pos;
+        AngleDeg rel_angle = rel_pos.th() - next_body;
 
-    target_rel_angle
-        = ServerParam::i().normalizeNeckAngle( target_rel_angle.degree() );
+        if ( rel_angle.abs() < ServerParam::i().maxNeckAngle() + next_view_width - 5.0 )
+        {
+            dlog.addText( Logger::ACTION,
+                          "%s:%d Neck_TurnToPoint (%.1f %.1f) rel_angle = %.1f"
+                          ,__FILE__, __LINE__,
+                          p->x, p->y, rel_angle.degree() );
+            return agent->doTurnNeck( rel_angle - agent->world().self().neck() );
+        }
+    }
 
-    return agent->doTurnNeck( target_rel_angle
-                              - agent->world().self().neck() );
+    dlog.addText( Logger::ACTION,
+                  "%s:%d Neck_TurnToPoint. cannot turn neck to target points. scan"
+                  ,__FILE__, __LINE__ );
+    Neck_ScanField().execute( agent );
+    return true;
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -188,37 +201,79 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
         return true;
     }
 
+    const Vector2D my_next = agent->effector().queuedNextMyPos();
+    const AngleDeg my_body_next = agent->effector().queuedNextMyBody();
+
     const Vector2D ball_next = agent->effector().queuedNextBallPos();
-    const AngleDeg ball_rel_angle_next
-        = agent->effector().queuedNextAngleFromBody( ball_next );
-#if 0
-    const ViewWidth next_vwidth = agent->effector().queuedNextViewWidth();
-    const double next_vwidth_value = next_vwidth.getWidth();
+    const AngleDeg ball_angle_next = ( ball_next - my_next ).th();
+    const AngleDeg ball_rel_angle_next = ball_angle_next - my_body_next;
+
+    const double next_view_width = agent->effector().queuedNextViewWidth().width();
 
     dlog.addText( Logger::ACTION,
                   "%s:%d: ball_next=(%.2f, %.2f), ball_rel_angle=%.1f, next_view=%.1f"
                   ,__FILE__, __LINE__,
                   ball_next.x, ball_next.y,
                   ball_rel_angle_next.degree(),
-                  next_vwidth_value );
-    // check if it is necessary to face neck to ball
-    if ( agent->world().ball().distFromSelf() > 30.0
-         || next_vwidth.type() == ViewWidth::WIDE )
+                  next_view_width );
+
+    // never look the ball
+    if ( ball_rel_angle_next.abs()
+         > ServerParam::i().maxNeckAngle() + next_view_width * 0.5 )
     {
-        double dir_diff = ( ball_rel_angle_next
-                            - agent->world().self().neck() ).abs();
-        // need to turn neck to ball
-        if ( dir_diff > next_vwidth_value * 0.5 - 5.0 )
-        {
-            agent->doTurnNeck( ball_rel_angle_next
-                               - agent->world().self().neck() );
-        }
+        Neck_ScanField().execute( agent );
+        return true;
     }
-    else
-#endif
+
+    // ball is near, directly look the ball
+    if ( my_next.dist( ball_next ) < 20.0 )
     {
         agent->doTurnNeck( ball_rel_angle_next
                            - agent->world().self().neck() );
+        return true;
+    }
+
+    double angle_buf = std::max( 0.0, next_view_width * 0.5 - 15.0 );
+
+    AngleDeg left_rel_angle = ball_rel_angle_next - angle_buf;
+    AngleDeg right_rel_angle = ball_rel_angle_next + angle_buf;
+
+    if ( left_rel_angle.isLeftOf( ServerParam::i().minNeckAngle() ) )
+    {
+        left_rel_angle = ServerParam::i().minNeckAngle();
+    }
+
+    if ( right_rel_angle.isRightOf( ServerParam::i().maxNeckAngle() ) )
+    {
+        right_rel_angle = ServerParam::i().maxNeckAngle();
+    }
+
+    int left_sum_count = 0;
+    int right_sum_count = 0;
+
+    agent->world().dirRangeCount( my_body_next + left_rel_angle,
+                                  next_view_width,
+                                  NULL, &left_sum_count, NULL );
+    agent->world().dirRangeCount( my_body_next + right_rel_angle,
+                                  next_view_width,
+                                  NULL, &right_sum_count, NULL );
+
+    dlog.addText( Logger::ACTION,
+                  "%s:%d: angle_buf=%.0f  left_rel=%.0f right_rel=%.0f"
+                  " left_sum=%d  right_sum=%d"
+                  ,__FILE__, __LINE__,
+                  angle_buf,
+                  left_rel_angle.degree(), right_rel_angle.degree(),
+                  left_sum_count, right_sum_count );
+
+
+    if ( left_sum_count > right_sum_count )
+    {
+        agent->doTurnNeck( left_rel_angle - agent->world().self().neck() );
+    }
+    else
+    {
+        agent->doTurnNeck( right_rel_angle - agent->world().self().neck() );
     }
 
     return true;
