@@ -370,7 +370,7 @@ Body_KickTwoStep::simulate_two_kick( Vector2D * achieved_vel,
     // first element is next kickable edge
     {
         const Ray2D desired_ray( ball_rpos, ( target_rpos - ball_rpos ).th() );
-        const Circle2D next_kickable_circle( my_next, my_kickable );
+        const Circle2D next_kickable_circle( my_next, my_kickable - 0.1 );
 
         // get intersection next kickable circle & desired ray
         // solutions are next ball pos, relative to current my pos
@@ -396,7 +396,7 @@ Body_KickTwoStep::simulate_two_kick( Vector2D * achieved_vel,
             required_vel = ( v1.r2() > v2.r2() ? v1 : v2 );
         }
 
-        if ( required_vel )
+        if ( required_vel.valid() )
         {
             double d = required_vel.r();
             double ball_noise = d * ServerParam::i().ballRand() * 1.412;
@@ -406,7 +406,7 @@ Body_KickTwoStep::simulate_two_kick( Vector2D * achieved_vel,
         }
 
         double opp_dist2 = DEFAULT_MIN_DIST2;
-        if ( required_vel
+        if ( required_vel.valid()
              && ! is_opp_kickable( agent, ball_rpos + required_vel, &opp_dist2 )
              && ( required_vel - ball_vel ).r() < max_accel )
         {
