@@ -46,13 +46,16 @@
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
 #include <rcsc/player/audio_sensor.h>
+#include <rcsc/player/say_message_builder.h>
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/geom/rect_2d.h>
 #include <rcsc/geom/sector_2d.h>
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
+#define DEBUG
+
 namespace rcsc {
 
 std::vector< Body_Pass::PassRoute > Body_Pass::S_cached_pass_route;
@@ -83,7 +86,7 @@ Body_Pass::execute( PlayerAgent * agent )
     double first_speed = 0.0;
     int receiver = 0;
 
-    if ( ! get_best_pass( agent->world(), &target_point, &first_speed, &receiver) )
+    if ( ! get_best_pass( agent->world(), &target_point, &first_speed, &receiver ) )
     {
         return false;
     }
@@ -117,8 +120,7 @@ Body_Pass::execute( PlayerAgent * agent )
     }
 
     if ( agent->config().useCommunication()
-         && 0 < receiver
-         && receiver < 12 )
+         && receiver != Unum_Unknown )
     {
         dlog.addText( Logger::ACTION,
                       "%s:%d: execute() set pass communication."
@@ -126,16 +128,10 @@ Body_Pass::execute( PlayerAgent * agent )
         Vector2D target_buf = target_point - agent->world().self().pos();
         target_buf.setLength( 1.0 );
 
-        std::string say_msg;
-        agent->audioSensor().encodePass( receiver,
-                                         target_point + target_buf,
-                                         say_msg );
-        if ( ! say_msg.empty() )
-        {
-            agent->doSay( say_msg );
-        }
-        //agent->effector().setPass(receiver, target_point + target_buf,
-        //agent->world().time() );
+        agent->addSayMessage( new PassMessage( receiver,
+                                               target_point + target_buf,
+                                               agent->effector().queuedNextBallPos(),
+                                               agent->effector().queuedNextBallVel() ) );
     }
 
     return true;
@@ -155,7 +151,7 @@ Body_Pass::get_best_pass( const WorldModel & world,
     static bool S_last_calc_valid = false;
     static Vector2D S_last_calc_target;
     static double S_last_calc_speed = 0.0;
-    static int S_last_calc_receiver = 0;
+    static int S_last_calc_receiver = Unum_Unknown;
 
     if ( S_last_calc_time == world.time() )
     {
@@ -349,7 +345,7 @@ Body_Pass::create_direct_pass( const WorldModel & world,
 #ifdef DEBUG
             dlog.addText( Logger::PASS,
                           "__ looks back pass. DF Line=%.1f. canceled",
-                          world.getDefenseLineX() );
+                          world.defenseLineX() );
 #endif
             return;
         }
@@ -389,7 +385,7 @@ Body_Pass::create_direct_pass( const WorldModel & world,
     {
         Vector2D fvel = receiver->vel();
         fvel /= ServerParam::i().defaultPlayerDecay();
-        fvel *= 3.0;
+        fvel *= std::min( receiver->velCount() + 1, 2 );
         base_player_pos += fvel;
     }
 
@@ -494,14 +490,14 @@ Body_Pass::create_direct_pass( const WorldModel & world,
     target_new = world.ball().pos();
     angle_new = receiver_angle;
     angle_new -= kickable_angle_buf;
-    target_new += Vector2D::polar2vector(receiver_dist, angle_new);
+    target_new += Vector2D::polar2vector( receiver_dist, angle_new );
 
     if ( verify_direct_pass( world,
                              receiver,
                              target_new,
                              receiver_dist,
                              angle_new,
-                             first_speed) )
+                             first_speed ) )
     {
         S_cached_pass_route
             .push_back( PassRoute( DIRECT,
@@ -513,6 +509,19 @@ Body_Pass::create_direct_pass( const WorldModel & world,
                                                          angle_new )
                                    )
                         );
+        dlog.addText( Logger::PASS,
+                      "Pass Success direct unum=%d pos=(%.1f %.1f). first_speed= %.1f",
+                      receiver->unum(),
+                      target_new.x, target_new.y,
+                      first_speed );
+    }
+    else
+    {
+        dlog.addText( Logger::PASS,
+                      "Pass Failed direct unum=%d pos=(%.1f %.1f). first_speed= %.1f",
+                      receiver->unum(),
+                      target_new.x, target_new.y,
+                      first_speed );
     }
 }
 
@@ -589,7 +598,7 @@ Body_Pass::create_lead_pass( const WorldModel & world,
     const AngleDeg receiver_angle = receiver_rel.th();
 
     const double circum = 2.0 * receiver_dist * M_PI;
-    const double angle_step_abs = std::max(5.0, 360.0 * ( 2.0 / circum ));
+    const double angle_step_abs = std::max( 5.0, 360.0 * ( 2.0 / circum ));
     /// angle loop
     double total_add_angle_abs;
     int count;
@@ -656,8 +665,8 @@ Body_Pass::create_lead_pass( const WorldModel & world,
 
             // check dir confidence
             int max_count = 100, ave_count = 100;
-            world.getDirRangeCount( target_angle - 10.0, target_angle + 10.0,
-                                    &max_count, &ave_count );
+            world.dirRangeCount( target_angle, 20.0,
+                                 &max_count, NULL, &ave_count );
             if ( max_count > 9 || ave_count > 3 )
             {
                 continue;
@@ -675,7 +684,12 @@ Body_Pass::create_lead_pass( const WorldModel & world,
                 continue;
             }
 
-            if ( target_point.x < receiver->pos().x - 3.0 )
+            if ( target_point.x < 0.0
+                 && target_point.x < receiver->pos().x - 3.0 )
+            {
+                continue;
+            }
+            if ( target_point.x < receiver->pos().x - 6.0 )
             {
                 continue;
             }
@@ -739,6 +753,22 @@ Body_Pass::create_lead_pass( const WorldModel & world,
                                            can_kick_by_one_step( world,
                                                                  first_speed,
                                                                  target_angle ) ) );
+                dlog.addText( Logger::PASS,
+                              "Pass Success lead unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f",
+                              receiver->unum(),
+                              target_point.x, target_point.y,
+                              target_angle.degree(),
+                              first_speed );
+            }
+            else
+            {
+                dlog.addText( Logger::PASS,
+                              "Pass Failed lead unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f",
+                              receiver->unum(),
+                              target_point.x, target_point.y,
+                              target_angle.degree(),
+                              first_speed );
+
             }
         }
     }
@@ -763,18 +793,18 @@ Body_Pass::create_through_pass(const WorldModel & world,
                                      ServerParam::i().pitchLength() - 6.0,
                                      ServerParam::i().pitchWidth() - 6.0 );
 
-    //static const double receiver_dash_speed = 0.9;
-    static const double receiver_dash_speed = 0.85;
+    static const double receiver_dash_speed = 1.0;
+    //static const double receiver_dash_speed = 0.85;
 
-    static const double S_min_dash = 10.0;
+    static const double S_min_dash = 5.0;
     static const double S_max_dash = 25.0;
     static const double S_dash_range = S_max_dash - S_min_dash;
-    static const double S_dash_inc = 5.0;
+    static const double S_dash_inc = 4.0;
     static const int S_dash_loop
         = static_cast< int >( std::ceil( S_dash_range / S_dash_inc ) ) + 1;
 
-    static const AngleDeg S_min_angle = -40.0;
-    static const AngleDeg S_max_angle = 40.0;
+    static const AngleDeg S_min_angle = -20.0;
+    static const AngleDeg S_max_angle = 20.0;
     static const double S_angle_range = ( S_min_angle - S_max_angle ).abs();
     static const double S_angle_inc = 10.0;
     static const int S_angle_loop
@@ -789,6 +819,14 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
     /////////////////////////////////////////////////////////////////
     // check receiver position
+    if ( receiver->pos().x > world.offsideLineX() - 0.5 )
+    {
+#ifdef DEBUG
+        dlog.addText( Logger::PASS,
+                      "__ receiver is offside" );
+#endif
+        return;
+    }
     if ( receiver->pos().x < world.self().pos().x - 10.0 )
     {
 #ifdef DEBUG
@@ -834,20 +872,19 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
     // angle loop
     AngleDeg dash_angle = S_min_angle;
-    for ( int i = 0; i < S_angle_loop; i++, dash_angle += S_angle_inc )
+    for ( int i = 0; i < S_angle_loop; ++i, dash_angle += S_angle_inc )
     {
-        const Vector2D base_dash = Vector2D::polar2vector( S_min_dash, dash_angle );
+        const Vector2D base_dash = Vector2D::polar2vector( 1.0, dash_angle );
 
         // dash dist loop
         double dash_dist = S_min_dash;
-        for ( int j = 0; j < S_dash_loop; j++, dash_dist += S_dash_inc )
+        for ( int j = 0; j < S_dash_loop; ++j, dash_dist += S_dash_inc )
         {
             Vector2D target_point = base_dash;
-            target_point /= S_min_dash;
             target_point *= dash_dist;
             target_point += receiver->pos();
 
-            if ( ! shrinked_pitch.contains(target_point) )
+            if ( ! shrinked_pitch.contains( target_point ) )
             {
                 // out of pitch
                 continue;
@@ -864,8 +901,8 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
             // check dir confidence
             int max_count = 100, ave_count = 100;
-            world.getDirRangeCount( target_angle - 10.0, target_angle + 10.0,
-                                    &max_count, &ave_count );
+            world.dirRangeCount( target_angle, 20.0,
+                                 &max_count, NULL, &ave_count );
             if ( max_count > 9 || ave_count > 3 )
             {
                 continue;
@@ -883,7 +920,7 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
             const double dash_step = dash_dist / receiver_dash_speed;// + 5.0;//+2.0
 
-            double end_speed = 0.5;//0.65
+            double end_speed = 0.81;//0.65
             double first_speed = 100.0;
             double ball_steps_to_target = 100.0;
             do
@@ -896,15 +933,16 @@ Body_Pass::create_through_pass(const WorldModel & world,
                 //= required_first_speed(target_dist,
                 //ServerParam::i().ballDecay(),
                 //end_speed);
-                if ( first_speed < ServerParam::i().ballSpeedMax() )
+                if ( first_speed > ServerParam::i().ballSpeedMax() )
                 {
-                    break;
+                    end_speed -= 0.1;
+                    continue;
                 }
 
                 ball_steps_to_target
                     = calc_length_geom_series( first_speed,
-                                                     target_dist,
-                                                     ServerParam::i().ballDecay() );
+                                               target_dist,
+                                               ServerParam::i().ballDecay() );
                 //= move_step_for_first( target_dist,
                 //first_speed,
                 //ServerParam::i().ballDecay() );
@@ -916,9 +954,10 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
                 end_speed -= 0.1;
             }
-            while ( end_speed > 0.2 );
+            while ( end_speed > 0.5 );
 
-            if ( first_speed > ServerParam::i().ballSpeedMax() )
+            if ( first_speed > ServerParam::i().ballSpeedMax()
+                 || dash_step > ball_steps_to_target )
             {
                 continue;
             }
@@ -932,9 +971,9 @@ Body_Pass::create_through_pass(const WorldModel & world,
             if ( verify_through_pass( world,
                                       receiver,
                                       receiver->pos(),
-                                      target_point, target_dist, target_rel.th(),
+                                      target_point, target_dist, target_angle,
                                       first_speed,
-                                      ball_steps_to_target) )
+                                      ball_steps_to_target ) )
             {
                 S_cached_pass_route
                     .push_back( PassRoute( THROUGH,
@@ -943,9 +982,28 @@ Body_Pass::create_through_pass(const WorldModel & world,
                                            first_speed,
                                            can_kick_by_one_step( world,
                                                                  first_speed,
-                                                                 target_rel.th() )
+                                                                 target_angle )
                                            )
                                 );
+                dlog.addText( Logger::PASS,
+                              "Pass Success through pass unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f dash_step=%.1f ball_step=%.1f",
+                              receiver->unum(),
+                              target_point.x, target_point.y,
+                              target_angle.degree(),
+                              first_speed,
+                              dash_step,
+                              ball_steps_to_target );
+            }
+            else
+            {
+                dlog.addText( Logger::PASS,
+                              "Pass Failed through unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f dash_step-%.1f ball_step=%.1f",
+                              receiver->unum(),
+                              target_point.x, target_point.y,
+                              target_angle.degree(),
+                              first_speed,
+                              dash_step,
+                              ball_steps_to_target );
             }
         }
     }
@@ -971,12 +1029,12 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
 
 #ifdef DEBUG
     dlog.addText( Logger::PASS,
-                  "____ verify direct pass to(%.1f %.1f). first_speed= %.3f. angle = %.1f",
+                  "____ verify direct pass to(%.1f %.1f). first_speed=%.3f. angle=%.1f",
                   target_point.x, target_point.y,
                   first_speed, target_angle.degree() );
 #endif
 
-#ifdef DEBUG
+#if 0
     const Interception interception( world.ball().pos() + first_vel,
                                      first_speed * ServerParam::i().ballDecay(),
                                      target_angle );
@@ -1003,29 +1061,31 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
           ++it )
     {
         if ( (*it)->posCount() > 10 ) continue;
+        if ( (*it)->isGhost() && (*it)->posCount() >= 4 ) continue;
 
         const double virtual_dash
-            = player_dash_speed * std::min( 5, (*it)->posCount() );
-#if 0
-        if ( (*it)->pos().dist( target_point ) - virtual_dash > target_dist + 2.0 )
-        {
-#ifdef DEBUG
-            dlog.addText( Logger::PASS,
-                          "______ opp%d(%.1f %.1f) is too far. not calculated.",
-                          (*it)->unum(),
-                          (*it)->pos().x, (*it)->pos().y );
-#endif
-            continue;
-        }
-#endif
+            = player_dash_speed * 0.8 * std::min( 5, (*it)->posCount() );
+
+//         if ( (*it)->pos().dist( target_point ) - virtual_dash > target_dist + 2.0 )
+//         {
+// #ifdef DEBUG
+//             dlog.addText( Logger::PASS,
+//                           "______ opp%d(%.1f %.1f) is too far. not calculated.",
+//                           (*it)->unum(),
+//                           (*it)->pos().x, (*it)->pos().y );
+// #endif
+//             continue;
+//         }
+
+
         if ( ( (*it)->angleFromSelf() - target_angle ).abs() > 100.0 )
         {
-#ifdef DEBUG
-            dlog.addText( Logger::PASS,
-                          "______ opp%d(%.1f %.1f) is back of target dir. not calculated.",
-                          (*it)->unum(),
-                          (*it)->pos().x, (*it)->pos().y );
-#endif
+// #ifdef DEBUG
+//             dlog.addText( Logger::PASS,
+//                           "______ opp%d(%.1f %.1f) is back of target dir. not calculated.",
+//                           (*it)->unum(),
+//                           (*it)->pos().x, (*it)->pos().y );
+//#endif
             continue;
         }
 
@@ -1041,7 +1101,7 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
             return false;
         }
 
-#ifdef DEBUG
+#if 0
         int cycle = static_cast< int >
             ( std::ceil( interception
                          .getReachCycle( (*it)->pos(),
@@ -1073,17 +1133,18 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
             {
 #ifdef DEBUG
                 dlog.addText( Logger::PASS,
-                              "______ opp%d(%.1f %.1f) is already on pass lien. rejected.",
+                              "______ opp%d(%.1f %.1f) can reach pass line. rejected. vdash=%.1f",
                               (*it)->unum(),
-                              (*it)->pos().x, (*it)->pos().y );
+                              (*it)->pos().x, (*it)->pos().y,
+                              virtual_dash );
 #endif
                 return false;
             }
 
             const double ball_steps_to_project
                 = calc_length_geom_series( next_speed,
-                                                 ball_to_opp.x,
-                                                 ServerParam::i().ballDecay() );
+                                           ball_to_opp.x,
+                                           ServerParam::i().ballDecay() );
             //= move_step_for_first(ball_to_opp.x,
             //next_speed,
             //ServerParam::i().ballDecay());
@@ -1121,7 +1182,7 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
   static method
 */
 bool
-Body_Pass::verify_through_pass( const WorldModel &world,
+Body_Pass::verify_through_pass( const WorldModel & world,
                                 const PlayerObject * receiver,
                                 const Vector2D & receiver_pos,
                                 const Vector2D & target_point,
@@ -1136,10 +1197,10 @@ Body_Pass::verify_through_pass( const WorldModel &world,
     const AngleDeg minus_target_angle = -target_angle;
     const double next_speed = first_speed * ServerParam::i().ballDecay();
 
-    const double receiver_to_target = receiver_pos.dist(target_point);
+    const double receiver_to_target = receiver_pos.dist( target_point );
 
 
-#ifdef DEBUG
+#if 0
     const Interception interception( world.ball().pos() + first_vel,
                                      first_speed * ServerParam::i().ballDecay(),
                                      target_angle );
@@ -1158,6 +1219,23 @@ Body_Pass::verify_through_pass( const WorldModel &world,
     }
 #endif
 
+    bool very_aggressive = false;
+    if ( target_point.x > 28.0
+         && target_point.x > world.ball().pos().x + 20.0 )
+    {
+        very_aggressive = true;
+    }
+    else if ( target_point.x > world.offsideLineX() + 15.0
+              && target_point.x > world.ball().pos().x + 15.0 )
+    {
+        very_aggressive = true;
+    }
+    else if ( target_point.x > 38.0
+              && target_point.x > world.offsideLineX()
+              && target_point.absY() < 14.0 )
+    {
+        very_aggressive = true;
+    }
 
     const PlayerPtrCont::const_iterator
         o_end = world.opponentsFromSelf().end();
@@ -1166,6 +1244,8 @@ Body_Pass::verify_through_pass( const WorldModel &world,
           it != o_end;
           ++it )
     {
+        if ( (*it)->posCount() > 10 ) continue;
+
         if ( (*it)->goalie() )
         {
             if ( target_point.absY() > ServerParam::i().penaltyAreaWidth() - 3.0
@@ -1175,7 +1255,7 @@ Body_Pass::verify_through_pass( const WorldModel &world,
             }
         }
 
-#ifdef DEBUG
+#if 0
         int cycle = static_cast< int >
             ( std::ceil( interception
                          .getReachCycle( (*it)->pos(),
@@ -1191,9 +1271,12 @@ Body_Pass::verify_through_pass( const WorldModel &world,
 #endif
 
         const double virtual_dash
-            = player_dash_speed * std::min(5, (*it)->posCount());
-        const double opp_to_target = (*it)->pos().dist(target_point);
-        if ( opp_to_target - virtual_dash < receiver_to_target + 2.0 )
+            = player_dash_speed * std::min( 2, (*it)->posCount() );
+        const double opp_to_target = (*it)->pos().dist( target_point );
+        double dist_rate = ( very_aggressive ? 0.8 : 1.0 );
+        double dist_buf = ( very_aggressive ? 0.5 : 2.0 );
+
+        if ( opp_to_target - virtual_dash < receiver_to_target * dist_rate + dist_buf )
         {
 #ifdef DEBUG
             dlog.addText( Logger::PASS,
@@ -1296,9 +1379,10 @@ Body_Pass::evaluate_routes( const WorldModel & world )
         double dir_conf_rate = 1.0;
         {
             AngleDeg pass_angle = ( it->receive_point_ - world.self().pos() ).th();
-            int dir_count = world.getDirRangeMaxCount( pass_angle - 10.0,
-                                                       pass_angle + 10.0 );
-            dir_conf_rate = std::pow( 0.95, dir_count );
+            int max_count = 0;
+            world.dirRangeCount( pass_angle, 20.0, &max_count, NULL, NULL );
+
+            dir_conf_rate = std::pow( 0.95, max_count );
         }
         //-----------------------------------------------------------
         double offense_rate
