@@ -2,7 +2,7 @@
 
 /*!
   \file body_shoot.h
-  \brief advanced shoot planning and behavior.
+  \brief advanced shoot planning and action.
 */
 
 /*
@@ -32,7 +32,7 @@
 #ifndef RCSC_ACTION_BODY_SHOOT_H
 #define RCSC_ACTION_BODY_SHOOT_H
 
-#include <rcsc/action/shoot_table.h>
+#include "shoot_table.h"
 
 #include <rcsc/player/soccer_action.h>
 
@@ -63,6 +63,10 @@ public:
     bool execute( PlayerAgent * agent );
 
 
+    /*!
+      \brief get the current shoot table
+      \return const reference to the shoot table instance.
+     */
     static
     const
     ShootTable & get_shoot_table( const PlayerAgent * agent )
