@@ -45,24 +45,37 @@ namespace rcsc {
 class IntentionKick
     : public SoccerIntention {
 private:
-    const Vector2D M_target_point; // global coordinate
-    const double M_first_speed; // first ball speed, when last kick
+    const Vector2D M_target_point; //!< target point
+    const double M_first_speed; //!< first ball speed when last kick
 
-    // player try to finish to kick within this step
+    //! player try to finish to kick within this step
     int M_kick_step;
 
-    // if this is true, player must finish kicks within M_kick_step
-    bool M_enforce_kick; // this is true, when clear, shoot, opp close case.
+    //! if this is true, player must finish kicks within M_kick_step
+    bool M_enforce_kick;
 
-    GameTime M_last_execute_time;
+    GameTime M_last_execute_time; //!< last executed time
 
 public:
+    /*!
+      \brief constructor
+      \param target_point kick target point
+      \param first_speed first ball speed when last kick
+      \param kick_step player try to finish to kick within this step
+      \param if this is true, player must finish kicks within M_kick_step
+      \param start_time action started time
+     */
     IntentionKick( const Vector2D & target_point,
                    const double & first_speed,
                    const int kick_step,
                    const bool enforce_kick,
                    const GameTime & start_time );
 
+    /*!
+      \brief check the status.
+      \param agent pointer to the agent itself.
+      \param true if action queue is finished.
+     */
     bool finished( const PlayerAgent * agent );
 
     /*!
@@ -75,14 +88,14 @@ public:
 
 private:
 
+    /*!
+      \brief clear the action queue
+     */
     void clear()
       {
           M_kick_step = 0;
       }
 
-
-    void addRelativeSubTarget( const Vector2D & target );
-
 };
 
 }
