@@ -107,9 +107,12 @@ IntentionTimeLimitAction::execute( PlayerAgent * agent )
     M_timer_count -= 1;
     M_last_execute_time = agent->world().time();
 
+    bool done_body = false;
+    bool done_neck = false;
+
     if ( M_body_action )
     {
-        M_body_action->execute( agent );
+        done_body = M_body_action->execute( agent );
     }
 
     if ( M_view_action )
@@ -119,7 +122,30 @@ IntentionTimeLimitAction::execute( PlayerAgent * agent )
 
     if ( M_neck_action )
     {
-        M_neck_action->execute( agent );
+        done_neck = M_neck_action->execute( agent );
+    }
+
+    if ( ! done_body )
+    {
+        std::cerr << agent->config().teamName()
+                  << ' ' << agent->world().self().unum()
+                  << " IntentionTimeLimitAction. no body action "
+                  << std::endl;
+        dlog.addText( Logger::ACTION,
+                      "%s:%d: error. no body action "
+                      ,__FILE__, __LINE__ );
+        return false;
+    }
+
+    if ( ! done_neck )
+    {
+        std::cerr << agent->config().teamName()
+                  << ' ' << agent->world().self().unum()
+                  << " IntentionTimeLimitAction. no neck action "
+                  << std::endl;
+        dlog.addText( Logger::ACTION,
+                      "%s:%d: error. no neck action "
+                      ,__FILE__, __LINE__ );
     }
 
     return true;
