@@ -49,23 +49,36 @@ class ViewAction;
 class IntentionTimeLimitAction
     : public SoccerIntention {
 private:
-    boost::shared_ptr< BodyAction > M_body_action;
+    boost::shared_ptr< BodyAction > M_body_action; //!< intended body action
 
-    boost::shared_ptr< NeckAction > M_neck_action;
+    boost::shared_ptr< NeckAction > M_neck_action; //!< intended neck action
 
-    boost::shared_ptr< ViewAction > M_view_action;
+    boost::shared_ptr< ViewAction > M_view_action; //!< intended view action
 
-    long M_timer_count;
+    long M_timer_count; //!< the size of remained queue
 
-    GameTime M_last_execute_time;
+    GameTime M_last_execute_time; //!< last executed time
 
 public:
+    /*!
+      \brief constructor
+      \param body_action dynamically allocated action object
+      \param neck_action dynamically allocated action object
+      \param view_action dynamically allocated action object
+      \param max_step the size of queue
+      \param start_time action started time
+     */
     IntentionTimeLimitAction( BodyAction * body_action,
                               NeckAction * neck_action,
                               ViewAction * view_action,
                               const long & max_step,
                               const GameTime & start_time );
 
+    /*!
+      \brief check the status.
+      \param agent pointer to the agent itself.
+      \param true if action queue is finished.
+     */
     bool finished( const PlayerAgent * agent );
 
     /*!
