@@ -480,12 +480,16 @@ CoachAgent::parse( const char * msg )
     }
     else if ( ! std::strncmp( msg, "(score ", 7 ) )
     {
-
+        analyzeScore( msg );
     }
     else if ( ! std::strncmp( msg, "(init ", 6 ) )
     {
         analyzeInit( msg );
     }
+    else if ( ! std::strncmp( msg, "(include ", 9 ) )
+    {
+        analyzeInclude( msg );
+    }
     else
     {
         std::cerr << config().teamName()
@@ -537,6 +541,14 @@ CoachAgent::analyzeInit( const char * msg )
     {
         doEye( true );
     }
+
+    // set compression level
+    if ( 0 < config().compression()
+         && config().compression() <= 9 )
+    {
+        CoachCompressionCommand com( config().compression() );
+        sendCommand( com );
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -583,8 +595,7 @@ CoachAgent::analyzeSeeGlobal( const char * msg )
                            M_impl->current_time_ );
 
     // update world model
-    if ( M_impl->visual_.time() == M_impl->current_time_
-         && world().time() != M_impl->current_time_ )
+    if ( M_impl->visual_.time() == M_impl->current_time_ )
     {
         M_worldmodel.updateAfterSeeGlobal( M_impl->visual_,
                                            M_impl->current_time_ );
@@ -698,9 +709,9 @@ CoachAgent::analyzeChangePlayerType( const char * msg )
     if ( std::sscanf( msg, "(ok change_player_type %d %d)",
                       &unum, &type ) == 2 )
     {
-        M_worldmodel.setPlayerType( world().ourSide(),
-                                    unum,
-                                    static_cast< HeteroID >( type ) );
+        M_worldmodel.recvChangePlayerType( world().ourSide(),
+                                           unum,
+                                           static_cast< HeteroID >( type ) );
     }
     else if ( std::sscanf( msg, "(change_player_type %d %d)",
                            &unum, &type ) == 2 )
@@ -715,7 +726,7 @@ CoachAgent::analyzeChangePlayerType( const char * msg )
         SideID opp_side = ( world().ourSide() == LEFT
                             ? RIGHT
                             : LEFT );
-        M_worldmodel.setPlayerType( opp_side, unum, Hetero_Unknown );
+        M_worldmodel.recvChangePlayerType( opp_side, unum, Hetero_Unknown );
     }
     else
     {
@@ -778,7 +789,10 @@ CoachAgent::analyzeServerParam( const char * msg )
 void
 CoachAgent::analyzeCLangVer( const char * msg )
 {
-    // M_clang_manager.parseCLangVer( msg );
+    //     std::cerr << config().teamName()
+    //               << " coach: "
+    //               << world().time()
+    //               << " recv: " << msg << std::endl;
 }
 
 /*-------------------------------------------------------------------*/
@@ -880,6 +894,20 @@ CoachAgent::analyzeTeamNames( const char * msg )
     }
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+CoachAgent::analyzeScore( const char * msg )
+{
+    std::cerr << config().teamName()
+              << " coach: "
+              << world().time()
+              << " recv " << msg << std::endl;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -910,6 +938,19 @@ CoachAgent::analyzeWarning( const char * msg )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+CoachAgent::analyzeInclude( const char * msg )
+{
+    std::cerr << config().teamName()
+              << " coach: "
+              << world().time()
+              << " recv " << msg << std::endl;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 CoachAgent::sendCommand( const CoachCommand & com )
@@ -1113,10 +1154,18 @@ CoachAgent::doSayFreeform( const std::string & msg )
 
     M_worldmodel.incFreeformSendCount();
 
-    std::ostringstream ostr;
-    ostr << "(say (freeform \"" << msg << "\"))";
+    //std::ostringstream ostr;
+    //ostr << "(say (freeform \"" << msg << "\"))";
+    //return ( M_client->sendMessage( ostr.str().c_str() ) > 0 );
 
-    return ( M_client->sendMessage( ostr.str().c_str() ) > 0 );
+    std::string freeform_msg;
+    freeform_msg.reserve( msg.length() + 32 );
+
+    freeform_msg = "(say (freeform \"";
+    freeform_msg += msg;
+    freeform_msg += "\"))";
+
+    return ( M_client->sendMessage( freeform_msg.c_str() ) > 0 );
 }
 
 }
