@@ -34,7 +34,7 @@
 
 #include <rcsc/coach/global_world_model.h>
 #include <rcsc/coach/coach_config.h>
-#include <rcsc/client/basic_client.h>
+#include <rcsc/common/soccer_agent.h>
 
 #include <boost/scoped_ptr.hpp>
 
@@ -46,8 +46,12 @@ struct CoachAgentImpl;
 class CoachCommand;
 class GlobalVisualSensor;
 
+/*!
+  \class CoachAgent
+  \brief abstract coach agent class
+ */
 class CoachAgent
-    : public BasicClient {
+    : public SoccerAgent {
 private:
 
     //! internal implementation object
@@ -97,62 +101,49 @@ public:
     const
     GlobalVisualSensor & visualSensor() const;
 
-    /*!
-      \brief finalize program process
-     */
-    void finalize()
-      {
-          handleExit();
-      }
-
 protected:
     /*!
       \brief analyze command line options
       \param cmd_parser command lien parser object
       \return only if "help" option is given, false is returned.
 
-      This method is called from BasicClient::init( argc, argv )
-      BasicClient::init(argc,argv) should be called in main().
-     */
-    virtual
-    bool doInit( CmdLineParser & cmd_parser );
-
-    /*!
-      \brief create connection to the server
-
-      This method is called on the top of BasicClient::run() method.
+      This method is called from SoccerAgent::init( argc, argv )
+      SoccerAgent::init(argc,argv) should be called in main().
       Do NOT call this method by yourself!
      */
     virtual
-    void doConnect();
+    bool initImpl( CmdLineParser & cmd_parser );
 
     /*!
-      \brief send init or reconnect command to server
+      \brief handle start event
+      \return status of start procedure.
 
-      init commad is sent in BasicClient's run() method
-      Do not call this method yourself!
-     */
+      This method is called on the top of BasicClient::run() method.
+      The concrete agent must connect to the server and send init command.
+      Do NOT call this method by yourself!
+    */
     virtual
-    void sendInitCommand();
-
-    /*!
-      \brief send disconnection command message to server
-      and set the server status to end.
-     */
-    virtual
-    void sendByeCommand();
+    bool handleStart();
 
     /*!
       \brief handle server message event
+
+      Do NOT call this method by yourself!
      */
     virtual
     void handleMessage();
 
     /*!
-      \brief handle select timeout
+      \brief handle timeout event
+      \param timeout_count count of timeout without sensory message.
+      \param waited_msec elapsed milli seconds sinc last sensory message.
+
+      This method is called when select() timeout occurs
+      Do NOT call this method by yourself!
      */
     virtual
-    void handleTimeout();
+    void handleTimeout( const int timeout_count,
+                        const int waited_msec );
 
     /*!
       \brief handle exit event
@@ -160,7 +151,30 @@ protected:
     virtual
     void handleExit();
 
+public:
+    /*!
+      \brief finalize program process
+     */
+    void finalize();
+
 private:
+
+    /*!
+      \brief send init or reconnect command to server
+
+      init commad is sent in BasicClient's run() method
+      Do not call this method yourself!
+     */
+    void sendInitCommand();
+
+    /*!
+      \brief send disconnection command message to server
+      and set the server status to end.
+     */
+    void sendByeCommand();
+
+    ////////////////////////////////////
+
     /*!
       \brief launch each parsing method depending on the message type
       \param msg raw server message
@@ -169,7 +183,7 @@ private:
 
     /*!
       \brief analyze init replay message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeInit( const char * msg );
 
@@ -184,61 +198,67 @@ private:
 
     /*!
       \brief analyze see_global message
-      \paramm msg raw server message
+      \param msg raw server message
      */
     void analyzeSeeGlobal( const char * msg );
 
     /*!
       \brief analyze hear message
-      \paramm msg raw server message
+      \param msg raw server message
      */
     void analyzeHear( const char * msg );
 
     /*!
       \brief analyze referee message
-      \paramm msg raw server message
+      \param msg raw server message
      */
     void analyzeHearReferee( const char * msg );
 
     /*!
       \brief analyze audio message from player
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearPlayer( const char * msg );
 
     /*!
       \brief analyze change_player_type message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeChangePlayerType( const char * msg );
 
     /*!
       \brief analyze player_type parameter message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzePlayerType( const char * msg );
 
     /*!
       \brief analyze player_param parameter message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzePlayerParam( const char * msg );
 
     /*!
       \brief analyze server_param parameter message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeServerParam( const char * msg );
 
     /*!
       \brief analyze clang version message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeCLangVer( const char * msg );
 
+    /*!
+      \brief analyze score message
+      \param msg raw server message
+    */
+    void analyzeScore( const char * msg );
+
     /*!
       \brief analyze ok message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeOK( const char * msg );
 
@@ -250,16 +270,22 @@ private:
 
     /*!
       \brief analyze error message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeError( const char * msg );
 
     /*!
       \brief analyze warning message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeWarning( const char * msg );
 
+    /*!
+      \brief analyze include message
+      \param msg raw server message
+    */
+    void analyzeInclude( const char * msg );
+
     /*!
       \brief main action decision. this method is called from handleMessage()
     */
@@ -328,7 +354,7 @@ protected:
       So, do *NOT* call this method by yourself.
     */
     virtual
-    void doAction() = 0;
+    void actionImpl() = 0;
 };
 
 }
