@@ -66,20 +66,24 @@ void
 CoachConfig::setDefaultParam()
 {
     // basic setting
-    M_team_name = "HELIOS";
+    M_team_name = "HELIOS2007";
     M_version = 9.4;
-    M_coach_name = "HELIOS_Coach";
 
+    M_coach_name = "HELIOS_Coach";
     M_use_coach_name = false;
 
-    M_interval_msec = 10;
+    M_interval_msec = 50;
     M_server_wait_seconds = 5;
 
     M_rcssserver_host = "localhost";
     M_rcssserver_port = 6002;
 
+    M_compression = -1;
+
     M_use_eye = true;
 
+    M_analyze_player_type = true;
+
     M_use_advise = true;
     M_use_freeform = true;
 
@@ -106,8 +110,12 @@ CoachConfig::createParamMap( ParamMap & param_map )
         ( "host", "h",  &M_rcssserver_host )
         ( "port", "", &M_rcssserver_port )
 
+        ( "compression", "", &M_compression )
+
         ( "use_eye", "", &M_use_eye )
 
+        ( "analyze_player_type", "", &M_analyze_player_type )
+
         ( "use_advise", "", &M_use_advise )
         ( "use_freeform", "", &M_use_freeform )
 
