@@ -348,12 +348,6 @@ public:
 
 };
 
-
-typedef std::vector< GlobalPlayerObject > GlobalPlayerVector;
-typedef std::list< GlobalPlayerObject > GlobalPlayerList;
-typedef std::vector< const GlobalPlayerObject * > GlobalPlayerPtrVector;
-
-
 } // namespace rcsc
 
 #endif
