@@ -123,7 +123,8 @@ GlobalVisualSensor::parseV7( const char * msg )
     }
     else
     {
-        std::cerr << "***ERROR*** invalide message(1) " << msg << std::endl;
+        std::cerr << __FILE__ << ' ' << __LINE__
+                  << "***ERROR*** invalide message(1) " << msg << std::endl;
         return;
     }
 
@@ -148,7 +149,8 @@ GlobalVisualSensor::parseV7( const char * msg )
                       &x, &y, &vx, &vy,
                       &n_read ) != 4 )
     {
-        std::cerr << "***ERROR*** invalide message(2) " << msg << ']'
+        std::cerr << __FILE__ << ' ' << __LINE__
+                  << "***ERROR*** invalide message(2) " << msg << ']'
                   << std::endl;
         return;
     }
@@ -176,11 +178,12 @@ GlobalVisualSensor::parseV7( const char * msg )
         int unum = 0;
 
         if ( std::sscanf( msg,
-                          " ((p %[^ ] %d %n ",
+                          " ((p %31[^ ] %d %n ",
                           teamname, &unum ,
                           &n_read ) != 2 )
         {
-            std::cerr << "***ERROR*** invalide message(3) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(3) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -206,7 +209,8 @@ GlobalVisualSensor::parseV7( const char * msg )
         }
         else
         {
-            std::cerr << "***ERROR*** invalide message(4) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(4) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -229,7 +233,8 @@ GlobalVisualSensor::parseV7( const char * msg )
                           " %lf %lf %lf %lf %lf %lf %n ",
                           &x, &y, &vx, &vy, &body, &neck, &n_read ) != 6 )
         {
-            std::cerr << "***ERROR*** invalide message(5) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(5) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -240,8 +245,11 @@ GlobalVisualSensor::parseV7( const char * msg )
 
         msg += n_read;
 
-        while ( *msg != ')' )
+        while ( *msg != '\0' && *msg != ')' )
         {
+            while ( *msg != '\0' && *msg == ' ' ) ++msg;
+            if ( *msg == '\0' ) break;
+
             if ( *msg == 't' )
             {
                 player.setTackle();
@@ -305,7 +313,8 @@ GlobalVisualSensor::parseV6( const char * msg )
     }
     else
     {
-        std::cerr << "***ERROR*** invalide message(1) " << msg << ']'
+        std::cerr << __FILE__ << ' ' << __LINE__
+                  << "***ERROR*** invalide message(1) " << msg << ']'
                   << std::endl;
         return;
     }
@@ -331,7 +340,8 @@ GlobalVisualSensor::parseV6( const char * msg )
                       &x, &y, &vx, &vy,
                       &n_read ) != 4 )
     {
-        std::cerr << "***ERROR*** invalide message(2) " << msg << ']'
+        std::cerr << __FILE__ << ' ' << __LINE__
+                  << "***ERROR*** invalide message(2) " << msg << ']'
                   << std::endl;
         return;
     }
@@ -359,11 +369,12 @@ GlobalVisualSensor::parseV6( const char * msg )
         int unum = 0;
 
         if ( std::sscanf( msg,
-                          " ((player %[^ ] %d %n ",
+                          " ((player %31[^ ] %d %n ",
                           teamname, &unum ,
                           &n_read ) != 2 )
         {
-            std::cerr << "***ERROR*** invalide message(3) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(3) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -389,7 +400,8 @@ GlobalVisualSensor::parseV6( const char * msg )
         }
         else
         {
-            std::cerr << "***ERROR*** invalide message(4) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(4) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -412,7 +424,8 @@ GlobalVisualSensor::parseV6( const char * msg )
                           " %lf %lf %lf %lf %lf %lf %n ",
                           &x, &y, &vx, &vy, &body, &neck, &n_read ) != 6 )
         {
-            std::cerr << "***ERROR*** invalide message(5) [" << msg << ']'
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << "***ERROR*** invalide message(5) [" << msg << ']'
                       << std::endl;
             break;
         }
@@ -451,7 +464,7 @@ GlobalVisualSensor::print( std::ostream & os ) const
     os << "Coach Visual " << M_time
        << " ball: " << M_ball.pos() << M_ball.vel() << std::endl;
 
-    for ( GlobalPlayerVector::const_iterator it = M_players.begin();
+    for ( std::vector< GlobalPlayerObject >::const_iterator it = M_players.begin();
           it != M_players.end();
           ++it )
     {
