@@ -37,8 +37,8 @@
 
 #include "global_visual_sensor.h"
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/geom/rect_2d.h>
 
 namespace rcsc {
@@ -49,7 +49,9 @@ namespace rcsc {
 */
 GlobalWorldModel::GlobalWorldModel()
     : M_time( -1, 0 )
+    , M_see_time( -1, 0 )
     , M_our_side( NEUTRAL )
+    , M_player_type_analyzer( *this )
     , M_change_count_left( 0 )
     , M_change_count_right( 0 )
     , M_last_playon_start( 0 )
@@ -119,9 +121,9 @@ GlobalWorldModel::setTeamName( const SideID side,
 
 */
 void
-GlobalWorldModel::setPlayerType( const SideID side,
-                                 const int unum,
-                                 const HeteroID type )
+GlobalWorldModel::recvChangePlayerType( const SideID side,
+                                        const int unum,
+                                        const HeteroID type )
 {
     if ( side == NEUTRAL )
     {
@@ -156,6 +158,12 @@ GlobalWorldModel::setPlayerType( const SideID side,
         }
     }
 
+    if ( side != ourSide()
+         && type == Hetero_Unknown )
+    {
+        M_player_type_analyzer.reset( unum );
+    }
+
 }
 
 /*-------------------------------------------------------------------*/
@@ -200,22 +208,61 @@ void
 GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
                                         const GameTime & current )
 {
+    if ( M_see_time.cycle() != current.cycle() - 1
+         && M_see_time.stopped() != current.stopped() - 1 )
+    {
+        // missed cycles??
+        if ( M_see_time.cycle() == current.cycle()
+             && M_see_time.stopped() > 0
+             && current.stopped() == 0 )
+        {
+            // back from the stopped mode
+        }
+        else
+        {
+            std::cerr << __FILE__ << ' ' << __LINE__
+                      << " missed cycles? last updated time = " << M_see_time
+                      << " current = " << current
+                      << std::endl;
+        }
+    }
+
     M_time = current;
 
+    if ( M_see_time == current )
+    {
+        return;
+    }
+
+    M_see_time = current;
+
+    // update team names
+
+    if ( ! see_global.teamNameLeft().empty() )
+    {
+        M_team_name_left = see_global.teamNameLeft();
+    }
+
+    if ( ! see_global.teamNameRight().empty() )
+    {
+        M_team_name_right = see_global.teamNameRight();
+    }
+
+    // update ball
+
     M_ball = see_global.ball();
 
-    M_players_left.clear();
-    M_players_right.clear();
+    // update player instance variables
 
-    GlobalPlayerList new_players;
+    std::list< GlobalPlayerObject > new_players;
 
-    const GlobalPlayerVector::const_iterator end = see_global.players().end();
-    for ( GlobalPlayerVector::const_iterator it = see_global.players().begin();
+    const std::vector< GlobalPlayerObject >::const_iterator end = see_global.players().end();
+    for ( std::vector< GlobalPlayerObject >::const_iterator it = see_global.players().begin();
           it != end;
           ++it )
     {
         bool found = false;
-        GlobalPlayerList::iterator p = M_players.begin();
+        std::list< GlobalPlayerObject >::iterator p = M_players.begin();
         while  ( p != M_players.end() )
         {
             if ( p->side() == it->side()
@@ -237,11 +284,16 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
     }
 
     M_players.clear();
-
     M_players.splice( M_players.end(), new_players );
 
-    const GlobalPlayerList::iterator pend = M_players.end();
-    for ( GlobalPlayerList::iterator p = M_players.begin();
+
+    // update the vector of player's pointer
+
+    M_players_left.clear();
+    M_players_right.clear();
+
+    const std::list< GlobalPlayerObject >::iterator pend = M_players.end();
+    for ( std::list< GlobalPlayerObject >::iterator p = M_players.begin();
           p != pend;
           ++p )
     {
@@ -254,6 +306,27 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
             M_players_right.push_back( &(*p) );
         }
     }
+
+
+    // analyzer player type
+
+    M_player_type_analyzer.update();
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        HeteroID id = M_player_type_analyzer.heteroID( unum );
+        if ( id != Hetero_Unknown )
+        {
+            if ( ourSide() == LEFT )
+            {
+                M_player_types_right[unum - 1] = id;
+            }
+            else
+            {
+                M_player_types_left[unum - 1] = id;
+            }
+        }
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -261,8 +334,8 @@ GlobalWorldModel::updateAfterSeeGlobal( const GlobalVisualSensor & see_global,
 
 */
 HeteroID
-GlobalWorldModel::getPlayerType( const SideID side,
-                                 const int unum ) const
+GlobalWorldModel::getHeteroID( const SideID side,
+                               const int unum ) const
 {
     if ( side == NEUTRAL )
     {
@@ -379,12 +452,12 @@ bool
 GlobalWorldModel::existKickablePlayer() const
 {
 
-    const GlobalPlayerList::const_iterator end = M_players.end();
-    for ( GlobalPlayerList::const_iterator it = M_players.begin();
+    const std::list< GlobalPlayerObject >::const_iterator end = M_players.end();
+    for ( std::list< GlobalPlayerObject >::const_iterator it = M_players.begin();
           it != end;
           ++it )
     {
-        HeteroID type = getPlayerType( it->side(), it->unum() );
+        HeteroID type = getHeteroID( it->side(), it->unum() );
         const PlayerType * param = PlayerTypeSet::i().get( type );
         double kickable_area = ( param
                                  ? param->kickableArea()
@@ -410,8 +483,8 @@ GlobalWorldModel::getPlayerNearestTo( const Vector2D & point ) const
     const GlobalPlayerObject * ptr = static_cast< GlobalPlayerObject * >( 0 );
     double max_dist2 = 20000.0;
 
-    const GlobalPlayerList::const_iterator end = M_players.end();
-    for ( GlobalPlayerList::const_iterator it = M_players.begin();
+    const std::list< GlobalPlayerObject >::const_iterator end = M_players.end();
+    for ( std::list< GlobalPlayerObject >::const_iterator it = M_players.begin();
           it != end;
           ++it )
     {
@@ -438,8 +511,8 @@ GlobalWorldModel::print( std::ostream & os ) const
 
     os << "Ball: " << M_ball.pos() << ' ' << M_ball.vel() << '\n';
 
-    const GlobalPlayerList::const_iterator end = players().end();
-    for ( GlobalPlayerList::const_iterator it = players().begin();
+    const std::list< GlobalPlayerObject >::const_iterator end = players().end();
+    for ( std::list< GlobalPlayerObject >::const_iterator it = players().begin();
           it != end;
           ++it )
     {
