@@ -306,6 +306,8 @@ const bool   ServerParam::DEFAULT_PEN_COACH_MOVES_PLAYERS = true;
 
 const std::string ServerParam::DEFAULT_MODULE_DIR = "";
 
+const double ServerParam::DEFAULT_BALL_STUCK_AREA = 3.0;
+
 /*
   It is necessary to check out whether server is NEW_QSTEP mode
   bool defined_qstep_l = false;
@@ -577,6 +579,7 @@ ServerParam::setDefaultParam()
 
     M_module_dir = DEFAULT_MODULE_DIR;
 
+    M_ball_stuck_area = DEFAULT_BALL_STUCK_AREA;
 
     setAdditionalParam();
 }
@@ -842,7 +845,11 @@ ServerParam::createMap()
         ( "pen_allow_mult_kicks", "", &M_pen_allow_mult_kicks )
         ( "pen_coach_moves_players", "", &M_pen_coach_moves_players )
 
-        ( "module_dir", "", &M_module_dir );
+        ( "module_dir", "", &M_module_dir )
+
+        ( "ball_stuck_area", "", &M_ball_stuck_area )
+        ( "coach_msg_file", "", &M_coach_msg_file )
+        ;
 }
 
 /*-------------------------------------------------------------------*/
@@ -1207,8 +1214,9 @@ ServerParam::convertFrom( const rcg::server_params_t & from )
 
     M_keepaway_length = rcg::nltohd( from.ka_length );
     M_keepaway_width = rcg::nltohd( from.ka_width );
+
+    M_ball_stuck_area = rcg::nltohd( from.ball_stuck_area );
     /*
-      from.sparelong5;
       from.sparelong6;
       from.sparelong7;
       from.sparelong8;
@@ -1371,8 +1379,9 @@ ServerParam::convertTo( rcg::server_params_t & to ) const
 
     to.ka_length = rcg::hdtonl( M_keepaway_length );
     to.ka_width = rcg::hdtonl( M_keepaway_width );
+
+    to.ball_stuck_area = rcg::hdtonl( M_ball_stuck_area );
     /*
-      from.sparelong5;
       from.sparelong6;
       from.sparelong7;
       from.sparelong8;
