@@ -442,8 +442,9 @@ FormationBPN::getPosition( const int unum,
                   << " *** ERROR *** FormationBPN::Param not found. unum = "
                   << unum
                   << std::endl;
-        return Vector2D( 0.0, 0.0 );
+        return Vector2D::INVALIDATED;
     }
+
     Formation::SideType type = Formation::SIDE;
     if ( M_mirror_number[unum - 1] > 0 )  type = Formation::MIRROR;
     if ( M_mirror_number[unum - 1] == 0 ) type = Formation::CENTER;
@@ -454,6 +455,22 @@ FormationBPN::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FormationBPN::getPositions( const Vector2D & focus_point,
+                            std::vector< Vector2D > & positions ) const
+{
+    positions.clear();
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        positions.push_back( getPosition( unum, focus_point ) );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FormationBPN::train( const std::list< Snapshot > & train_data )
