@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*!
-	\file formation_delaunay.cpp
+	\file formation_dt.cpp
 	\brief Delaunay Triangulation formation data classes Source File.
 */
 
@@ -46,8 +46,8 @@ namespace rcsc {
 
 */
 FormationDT::Param::Param( const Snapshot & data )
-    : M_ball( data.ball_ )
-    , M_players( data.players_ )
+    : ball_( data.ball_ )
+    , players_( data.players_ )
 {
 
 }
@@ -67,7 +67,7 @@ FormationDT::Param::getPosition( const int unum ) const
         return Vector2D( 0.0, 0.0 );
     }
 
-    return M_players[unum - 1];
+    return players_[unum - 1];
 }
 
 /*-------------------------------------------------------------------*/
@@ -97,6 +97,49 @@ Formation::Snapshot
 FormationDT::createDefaultParam()
 {
 #if 1
+    // 4-3-3
+
+    // 1: goalie
+    // 2: left center back
+    // 3(2): right center back
+    // 4: left side back
+    // 5(4): right side back
+    // 6: defensive half
+    // 7: left offensive half
+    // 8(7): left side half
+    // 9(8): right side half
+    // 10: left forward
+    // 11(10): right forward
+    createNewRole( 1, "Goalie", Formation::CENTER );
+    createNewRole( 2, "CenterBack", Formation::SIDE );
+    setMirrorType( 3, 2 );
+    createNewRole( 4, "SideBack", Formation::SIDE );
+    setMirrorType( 5, 4 );
+    createNewRole( 6, "DefensiveHalf", Formation::CENTER );
+    createNewRole( 7, "OffensiveHalf", Formation::SIDE );
+    setMirrorType( 8, 7 );
+    createNewRole( 9, "SideForward", Formation::SIDE );
+    setMirrorType( 10, 9 );
+    createNewRole( 11, "CenterForward", Formation::CENTER );
+
+    Snapshot snap;
+
+    snap.ball_.assign( 0.0, 0.0 );
+    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+
+#elif 0
+    // 1-3-4-2
+
     // 1: goalie
     // 2: sweeper
     // 3: left side back
@@ -111,14 +154,14 @@ FormationDT::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setRoleName( 4, getRoleName( 3 ) ); setMirrorType( 4, 3 );
+    setMirrorType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::CENTER );
     createNewRole( 6, "OffensiveHalf", Formation::SIDE );
-    setRoleName( 7, getRoleName( 6 ) ); setMirrorType( 7, 6 );
+    setMirrorType( 7, 6 );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setRoleName( 9, getRoleName( 8 ) ); setMirrorType( 9, 8 );
+    setMirrorType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setRoleName( 11, getRoleName( 10 ) ); setMirrorType( 11, 10 );
+    setMirrorType( 11, 10 );
 
     Snapshot snap;
 
@@ -135,6 +178,8 @@ FormationDT::createDefaultParam()
     snap.players_.push_back( Vector2D( 10.0, -10.0 ) );
     snap.players_.push_back( Vector2D( 10.0, 10.0 ) );
 #else
+    // 3-5-2
+
     // 1: goalie
     // 2: sweeper
     // 3: left side back
@@ -255,33 +300,63 @@ FormationDT::createNewRole( const int unum,
 */
 Vector2D
 FormationDT::getPosition( const int unum,
-                          const Vector2D & ball_pos ) const
+                          const Vector2D & focus_point ) const
 {
     if ( unum < 1 || 11 < unum )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " *** ERROR *** invalid unum " << unum
                   << std::endl;
-        return Vector2D( 0.0, 0.0 );
+        return Vector2D::INVALIDATED;
     }
 
     DelaunayTriangulation::TrianglePtr tri
-        = M_triangulation.findTriangleContains( ball_pos );
+        = M_triangulation.findTriangleContains( focus_point );
 
+    // linear interpolation
+    return interpolate( unum, focus_point, tri );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+FormationDT::getPositions( const Vector2D & focus_point,
+                           std::vector< Vector2D > & positions ) const
+{
+    positions.clear();
+
+    DelaunayTriangulation::TrianglePtr tri
+        = M_triangulation.findTriangleContains( focus_point );
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        positions.push_back( interpolate( unum, focus_point, tri ) );
+    }
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+Vector2D
+FormationDT::interpolate( const int unum,
+                          const Vector2D & focus_point,
+                          const DelaunayTriangulation::TrianglePtr tri ) const
+{
     if ( ! tri )
     {
-        // no triangle!!
-        // search the nearest vertex
-
         const DelaunayTriangulation::Vertex * v
-            = M_triangulation.findNearestVertex( ball_pos );
+            = M_triangulation.findNearestVertex( focus_point );
 
         if ( ! v )
         {
-            //std::cerr << __FILE__ << ":" << __LINE__
-            //          << " *** ERROR *** No vertex!"
-            //          << std::endl;
-            return Vector2D( 0.0, 0.0 );
+            std::cerr << __FILE__ << ":" << __LINE__
+                      << " *** ERROR *** No vertex!"
+                      << std::endl;
+            return Vector2D::INVALIDATED;
         }
 
         try
@@ -294,11 +369,10 @@ FormationDT::getPosition( const int unum,
             std::cerr << __FILE__ << ":" << __LINE__
                       << " exception caught! "
                       << e.what() << std::endl;
-            return Vector2D( 0.0, 0.0 );
+            return Vector2D::INVALIDATED;
         }
     }
 
-    // linear interpolation
     try
     {
         Vector2D result_0 = M_param.at( tri->vertex( 0 )->id() ).getPosition( unum );
@@ -306,23 +380,23 @@ FormationDT::getPosition( const int unum,
         Vector2D result_2 = M_param.at( tri->vertex( 2 )->id() ).getPosition( unum );
 
         Line2D line_0( tri->vertex( 0 )->pos(),
-                       ball_pos );
+                       focus_point );
 
         Segment2D segment_12( tri->vertex( 1 )->pos(),
                               tri->vertex( 2 )->pos() );
 
         Vector2D intersection_12 = segment_12.intersection( line_0 );
 
-        if ( ! intersection_12 )
+        if ( ! intersection_12.valid() )
         {
-            if ( ball_pos.dist2( tri->vertex( 0 )->pos() ) < 1.0e-5 )
+            if ( focus_point.dist2( tri->vertex( 0 )->pos() ) < 1.0e-5 )
             {
                 return result_0;
             }
 
             std::cerr << __FILE__ << ":" << __LINE__
                       << "***ERROR*** No intersection!\n"
-                      << " ball=" << ball_pos
+                      << " focus=" << focus_point
                       << " line_intersection=" << intersection_12
                       << "\n v0=" << tri->vertex( 0 )->pos()
                       << " v1=" << tri->vertex( 1 )->pos()
@@ -342,9 +416,9 @@ FormationDT::getPosition( const int unum,
             + ( result_2 - result_1 ) * ( dist_1i / ( dist_1i + dist_2i ) );
 
         // distance from vertex_0 to ball
-        double dist_0b = tri->vertex( 0 )->pos().dist( ball_pos );
+        double dist_0b = tri->vertex( 0 )->pos().dist( focus_point );
         // distance from interxectin_12 to ball
-        double dist_ib = intersection_12.dist( ball_pos );
+        double dist_ib = intersection_12.dist( focus_point );
 
         // interpolation result of vertex_0 & intersection_12
         Vector2D result_012
@@ -359,7 +433,7 @@ FormationDT::getPosition( const int unum,
                   << " Exception caught!! "
                   << e.what()
                   << std::endl;
-        return  Vector2D( 0.0, 0.0 );
+        return  Vector2D::INVALIDATED;
     }
 }
 
@@ -448,7 +522,7 @@ FormationDT::read( std::istream & is )
               it != M_param.end();
               ++it )
         {
-            M_triangulation.addVertex( it->M_ball );
+            M_triangulation.addVertex( it->ball_ );
         }
 
         M_triangulation.compute();
@@ -552,7 +626,7 @@ FormationDT::readSamples( std::istream & is )
         }
         msg += n_read;
 
-        M_param.back().M_ball.assign( read_x, read_y );
+        M_param.back().ball_.assign( read_x, read_y );
 
         for ( int unum = 1; unum <= 11; ++unum )
         {
@@ -569,7 +643,7 @@ FormationDT::readSamples( std::istream & is )
             }
             msg += n_read;
 
-            M_param.back().M_players[unum - 1].assign( read_x, read_y );
+            M_param.back().players_[unum - 1].assign( read_x, read_y );
         }
     }
 
@@ -597,12 +671,12 @@ FormationDT::print( std::ostream & os ) const
           it != M_param.end();
           ++it )
     {
-        os << it->M_ball.x << ' '
-           << it->M_ball.y << ' ';
+        os << it->ball_.x << ' '
+           << it->ball_.y << ' ';
 
         for ( std::vector< Vector2D >::const_iterator p
-                  = it->M_players.begin();
-              p != it->M_players.end();
+                  = it->players_.begin();
+              p != it->players_.end();
               ++p )
         {
             os << p->x << ' '
