@@ -52,7 +52,7 @@ const double FormationRBF::Param::PITCH_WIDTH = 68.0 + 10.0;
 
 */
 FormationRBF::Param::Param()
-    : M_net()
+    : M_net( 2, 2 )
 {
 
 }
@@ -65,23 +65,15 @@ Vector2D
 FormationRBF::Param::getPosition( const Vector2D & ball_pos,
                                   const Formation::SideType type ) const
 {
-    RBFNetwork::input_vector input;
+    RBFNetwork::input_vector input( 2, 0.0 );
 
     input[0] = ball_pos.x;
-    //bound( 0.0,
-    //       ball_pos.x / PITCH_LENGTH + 0.5,
-    //       1.0 );
     input[1] = ball_pos.y;
-    //bound( 0.0,
-    //       ball_pos.y / PITCH_WIDTH + 0.5,
-    //       1.0 );
 
     RBFNetwork::output_vector output;
 
     M_net.propagate( input, output );
 
-    //return Vector2D( ( bound( 0.0, output[0], 1.0 ) - 0.5 ) * PITCH_LENGTH,
-    //                 ( bound( 0.0, output[1], 1.0 ) - 0.5 ) * PITCH_WIDTH );
     return Vector2D( bound( -PITCH_LENGTH*0.5, output[0], PITCH_LENGTH*0.5 ),
                      bound( -PITCH_WIDTH*0.5, output[1], PITCH_WIDTH*0.5 ) );
 }
@@ -425,7 +417,7 @@ FormationRBF::getPosition( const int unum,
                   << " *** ERROR *** FormationRBF::Param not found. unum = "
                   << unum
                   << std::endl;
-        return Vector2D( 0.0, 0.0 );
+        return Vector2D::INVALIDATED;
     }
     Formation::SideType type = Formation::SIDE;
     if ( M_mirror_number[unum - 1] > 0 )  type = Formation::MIRROR;
@@ -437,6 +429,22 @@ FormationRBF::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FormationRBF::getPositions( const Vector2D & focus_point,
+                            std::vector< Vector2D > & positions ) const
+{
+    positions.clear();
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        positions.push_back( getPosition( unum, focus_point ) );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FormationRBF::train( const std::list< Snapshot > & train_data )
@@ -485,14 +493,8 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
                 std::cerr << "FormationRBF::train. added new center "
                           << snap->ball_
                           << std::endl;
-                RBFNetwork::input_vector center;
-                //center[0] = bound( 0.0,
-                //                   snap->ball_.x / PITCH_LENGTH + 0.5,
-                //                   1.0 );
+                RBFNetwork::input_vector center( 2, 0.0 );
                 center[0] = snap->ball_.x;
-                //center[1] = bound( 0.0,
-                //                   snap->ball_.y / PITCH_WIDTH + 0.5,
-                //                   1.0 );
                 center[1] = snap->ball_.y;
                 net.addCenter( center );
 
@@ -502,8 +504,8 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
 
         net.printUnits( std::cerr );
 
-        RBFNetwork::input_vector input;
-        RBFNetwork::output_vector teacher;
+        RBFNetwork::input_vector input( 2, 0.0 );
+        RBFNetwork::output_vector teacher( 2, 0.0 );
 
         const std::list< Formation::Snapshot >::const_iterator snap_end
             = train_data.end();
@@ -521,20 +523,6 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
                   snap != snap_end;
                   ++snap, snap_count += 1.0 )
             {
-                /*
-                input[0] = bound( 0.0,
-                                  snap->ball_.x / PITCH_LENGTH + 0.5,
-                                  1.0 );
-                input[1] = bound( 0.0,
-                                  snap->ball_.y / PITCH_WIDTH + 0.5,
-                                  1.0 );
-                teacher[0] = bound( 0.0,
-                                    snap->players_[unum - 1].x / PITCH_LENGTH + 0.5,
-                                    1.0 );
-                teacher[1] = bound( 0.0,
-                                    snap->players_[unum - 1].y / PITCH_WIDTH + 0.5,
-                                    1.0 );
-                */
                 input[0] = snap->ball_.x;
                 input[1] = snap->ball_.y;
                 teacher[0] = snap->players_[unum - 1].x;
