@@ -35,7 +35,6 @@
 
 #include "formation_uva.h"
 
-#include <rcsc/common/server_param.h>
 #include <rcsc/math_util.h>
 
 #include <cstdio>
@@ -257,7 +256,7 @@ FormationUvA::getPosition( const int unum,
         std::cerr << __FILE__ << ':' << __LINE__
                   << " **ERROR*** Invalid player number " << unum
                   << std::endl;
-        return Vector2D( 0.0, 0.0 );
+        return Vector2D::INVALIDATED;
     }
 
     Vector2D home_pos = M_home_pos[unum - 1];
@@ -286,9 +285,12 @@ FormationUvA::getPosition( const int unum,
                         home_pos.x,
                         it->second.maxX() );
 
-    home_pos.y = bound( - ServerParam::i().pitchHalfWidth() * maxYPercentage(),
+//     home_pos.y = bound( - ServerParam::i().pitchHalfWidth() * maxYPercentage(),
+//                         home_pos.y,
+//                         + ServerParam::i().pitchHalfWidth() * maxYPercentage() );
+    home_pos.y = bound( - 52.5 * maxYPercentage(),
                         home_pos.y,
-                        + ServerParam::i().pitchHalfWidth() * maxYPercentage() );
+                        + 52.5 * maxYPercentage() );
 
     return home_pos;
 }
@@ -296,6 +298,22 @@ FormationUvA::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FormationUvA::getPositions( const Vector2D & focus_point,
+                            std::vector< Vector2D > & positions ) const
+{
+    positions.clear();
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        positions.push_back( getPosition( unum, focus_point ) );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FormationUvA::train( const std::list< Snapshot > & train_data )
