@@ -8,7 +8,7 @@
 /*
  *Copyright:
 
- Copyright (C) Hidehisa Akiyama
+ Copyright (C) Hidehisa Akiyama, Hiroki Shimora
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -52,6 +52,8 @@ private:
     //! not used
     Segment2D();
 
+    bool checkIntersectsOnLine( const Vector2D & p ) const;
+
 public:
     /*!
       \brief construct from 2 points
@@ -200,6 +202,50 @@ public:
     */
     Vector2D intersection( const Line2D & other ) const;
 
+    /*!
+      \brief check if segments cross each other or not.
+      \param other segment for cross checking
+      \return true if this segment crosses, otherwise returns false.
+    */
+    bool intersects( const Segment2D & other ) const;
+
+    /*!
+      \brief check if segments intersect each other on non terminal point.
+      \param other segment for cross checking
+      \return true if this segments intersect and intersection point is not a
+      terminal point of segment.
+      false if segments not intersect or intersect on terminal point of segment.
+    */
+    bool intersectsExceptTerminalPoint( const Segment2D & other ) const;
+
+    /*!
+      \brief get a point on segment where distance of point is minimal.
+      \param p point
+      \return nearest point on segment. if multiple nearest points found.
+       returns one of them.
+    */
+    Vector2D nearestPoint( const Vector2D & p ) const;
+
+    /*!
+      \brief get minimum distance between this segment and point
+      \param p point
+      \return minimum distance between this segment and point
+    */
+    double distance( const Vector2D & p ) const;
+
+    /*!
+      \brief get minimum distance between 2 segments
+      \param seg segment
+      \return minimum distance between 2 segments
+    */
+    double distance( const Segment2D & seg ) const;
+
+    /*!
+      \brief get maximum distance between this segment and point
+      \param p point
+      \return maximum distance between this segment and point
+    */
+    double farthestDistance( const Vector2D & p ) const;
 };
 
 }
