@@ -41,12 +41,25 @@
 #include <cstring> // memset(), memcpy()
 #include <cerrno> // errno
 
+#ifdef HAVE_NETDB_H
 #include <netdb.h> // gethostbyname(), getaddrinfo(), freeaddrinfo()
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h> // close()
+#endif
 #include <fcntl.h> // fcntl()
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h> // socket(), getaddrinfo(), freeaddrinfo()
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h> // socket(), getaddrinfo(), freeaddrinfo()
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h> // inet_addr()
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h> // struct sockaddr_in, struct in_addr, htons
+#endif
 
 namespace rcsc {
 
@@ -119,13 +132,17 @@ UDPSocket::~UDPSocket()
 bool
 UDPSocket::open()
 {
+#ifdef HAVE_SOCKET
     // create socket
     M_fd = ::socket( AF_INET, SOCK_DGRAM, 0 );
+#endif
 
     if ( fd() == -1 )
     {
         std::cerr << "UDPSocket::connect() failed to open a socket."
                   << std::endl;
+        perror( "socket" );
+        std::cerr << errno << std::endl;
         return false;
     }
 
@@ -181,37 +198,57 @@ UDPSocket::setAddr( const char * hostname,
     hints.ai_protocol = 0;
 
     struct addrinfo * res;
-    if ( ::getaddrinfo( hostname, NULL, &hints, &res ) == 0 )
+    int err = ::getaddrinfo( hostname, NULL, &hints, &res );
+    if ( err != 0 )
     {
-        //std::cerr << "UDPSocket::connect(): created by getaddrinfo()"
-        //          << std::endl;
-        M_dest->addr_.sin_addr.s_addr
-            = (reinterpret_cast< struct sockaddr_in * >(res->ai_addr))->sin_addr.s_addr;
-        M_dest->addr_.sin_family = AF_INET;
-        M_dest->addr_.sin_port = htons( port );
+        std::cerr << "UDPSocket::connect() failed to find host [" << hostname
+                  << "]" << std::endl;
+        std::cerr << "UDPSocket. error " << err
+                  << gai_strerror( err ) << std::endl;
+        this->close();
+        return false;
 
-        freeaddrinfo( res );
-        return true;
     }
+
+    M_dest->addr_.sin_addr.s_addr
+        = (reinterpret_cast< struct sockaddr_in * >(res->ai_addr))->sin_addr.s_addr;
+    M_dest->addr_.sin_family = AF_INET;
+    M_dest->addr_.sin_port = htons( port );
+
+    freeaddrinfo( res );
+    return true;
+
 #endif
 
 #ifdef HAVE_GETHOSTBYNAME
-    struct hostent * host_entry = ::gethostbyname( hostname );
-    if ( host_entry )
+#ifdef HAVE_INET_ADDR
+    M_dest->addr_.sin_addr.s_addr = inet_addr( hostname );
+    if ( M_dest->addr_.sin_addr.s_addr == 0xffffffff )
+#endif
     {
-        //std::cerr << "UDPSocket::connect(): created by gethostbyname()"
-        //          << std::endl;
+        struct hostent * host_entry = ::gethostbyname( hostname );
+        if ( ! host_entry )
+        {
+            std::cerr << hstrerror( h_errno ) << std::endl;
+            std::cerr << "UDPSocket::connect() host not found ["
+                      << hostname << "]" << std::endl;
+            this->close();
+            return false;
+        }
+
         std::memcpy( &(M_dest->addr_.sin_addr.s_addr),
                      host_entry->h_addr_list[0],
                      host_entry->h_length );
-        M_dest->addr_.sin_family = AF_INET;
-        M_dest->addr_.sin_port = htons( port );
-
-        return true;
     }
+
+    M_dest->addr_.sin_family = AF_INET;
+    M_dest->addr_.sin_port = htons( port );
+
+    return true;
 #endif
 
-    std::cerr << "UDPSocket::connect() failed to find host [" << hostname
+    std::cerr << "UDPSocket::connect() no getaddrinfo or gethostbyname."
+              << "failed to find host [" << hostname
               << "]" << std::endl;
     this->close();
     return false;
