@@ -39,18 +39,34 @@
 
 namespace rcsc {
 
+/*!
+  \struct LongNamePredicate
+  \brief function object to check if the parameter's long name is same
+  or not.
+ */
 struct LongNamePredicate
     : public std::unary_function< ParamPtr, bool > {
-    std::string name_;
+
+    std::string name_; //!< name string to be compared
+
+    /*!
+      \brief construct with the compared long name string
+      \param long_name the compared long name
+     */
+    explicit
     LongNamePredicate( const std::string & long_name )
         : name_( long_name )
       { }
 
+    /*!
+      \brief predicate operator
+      \param arg compared parameter
+      \return compared result
+     */
     result_type operator()( const argument_type & arg ) const
       {
           return arg->longName() == name_;
       }
-
 };
 
 namespace {
@@ -274,7 +290,7 @@ ParamMap::remove( const std::string & long_name )
         = M_long_name_map.find( long_name );
     if ( it_long != M_long_name_map.end() )
     {
-        if ( it_long->second->shortName().empty() )
+        if ( ! it_long->second->shortName().empty() )
         {
             std::map< std::string, ParamPtr >::iterator it_short
                 = M_short_name_map.find( it_long->second->shortName() );
