@@ -48,8 +48,13 @@ namespace rcsc {
   \brief wrapper for bool switch.
  */
 struct BoolSwitch {
-    bool * ptr_;
+    bool * ptr_; //!< raw pointer to the parameter variable
 
+    /*!
+      \param constructor
+      \param ptr raw pointer to the parameter variable.
+     */
+    explicit
     BoolSwitch( bool * ptr )
         : ptr_( ptr )
       {
@@ -155,6 +160,9 @@ public:
 
 };
 
+/*!
+  \brief type of ParamEntry pointer
+ */
 typedef boost::shared_ptr< ParamEntry > ParamPtr;
 
 
@@ -330,7 +338,7 @@ class ParamMap {
 private:
 
     /*!
-      \class Registrer
+      \class Registrar
       \brief convinience class to simplify parameter registration
      */
     class Registrar {
