@@ -40,6 +40,7 @@
 #include "logger.h"
 #include "body_sensor.h"
 #include "player_command.h"
+#include "say_message_builder.h"
 
 #include <rcsc/math_util.h>
 #include <rcsc/common/server_param.h>
@@ -50,8 +51,8 @@ namespace rcsc {
 /*!
 
 */
-ActionEffector::ActionEffector( const WorldModel & world )
-    : M_world( world )
+ActionEffector::ActionEffector( const PlayerAgent & agent )
+    : M_agent( agent )
     , M_command_body( static_cast< PlayerBodyCommand * >( 0 ) )
     , M_command_turn_neck( static_cast< PlayerTurnNeckCommand * >( 0 ) )
     , M_command_change_view( static_cast< PlayerChangeViewCommand * >( 0 ) )
@@ -73,6 +74,7 @@ ActionEffector::ActionEffector( const WorldModel & world )
     , M_tackle_power( 0.0 )
     , M_tackle_dir( 0.0 )
     , M_turn_neck_moment( 0.0 )
+    , M_protocol_version( 9.0 )
     , M_say_message( "" )
     , M_pointto_pos( 0.0, 0.0 )
 {
@@ -195,7 +197,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.kickCount() != M_command_counter[PlayerCommand::KICK] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost kick? at " << M_last_action_time
                   << " sense=" << sense.kickCount()
                   << " internal=" << M_command_counter[PlayerCommand::KICK]
@@ -212,7 +214,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.turnCount() != M_command_counter[PlayerCommand::TURN] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost turn? at " << M_last_action_time
                   << " sense=" << sense.turnCount()
                   << " internal=" << M_command_counter[PlayerCommand::TURN]
@@ -229,7 +231,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.dashCount() != M_command_counter[PlayerCommand::DASH] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost dash? at " << M_last_action_time
                   << " sense=" << sense.dashCount()
                   << " internal=" << M_command_counter[PlayerCommand::DASH]
@@ -247,7 +249,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.moveCount() != M_command_counter[PlayerCommand::MOVE] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost move? at " << M_last_action_time
                   << " sense=" << sense.moveCount()
                   << " internal=" << M_command_counter[PlayerCommand::MOVE]
@@ -263,7 +265,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.catchCount() != M_command_counter[PlayerCommand::CATCH] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost catch? at " << M_last_action_time
                   << " sense=" << sense.catchCount()
                   << " internal=" << M_command_counter[PlayerCommand::CATCH]
@@ -279,7 +281,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.tackleCount() != M_command_counter[PlayerCommand::TACKLE] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost tackle? at " << M_last_action_time
                   << " sense=" << sense.tackleCount()
                   << " internal=" << M_command_counter[PlayerCommand::TACKLE]
@@ -296,7 +298,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.turnNeckCount() != M_command_counter[PlayerCommand::TURN_NECK] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost turn_neck? at " << M_last_action_time
                   << " sense=" << sense.turnNeckCount()
                   << " internal=" << M_command_counter[PlayerCommand::TURN_NECK]
@@ -312,7 +314,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.changeViewCount() != M_command_counter[PlayerCommand::CHANGE_VIEW] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost change_view? at " << M_last_action_time
                   << " sense=" << sense.changeViewCount()
                   << " internal=" << M_command_counter[PlayerCommand::CHANGE_VIEW]
@@ -326,7 +328,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.sayCount() != M_command_counter[PlayerCommand::SAY] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost say? at " << M_last_action_time
                   << " sense=" << sense.sayCount()
                   << " internal=" << M_command_counter[PlayerCommand::SAY]
@@ -340,7 +342,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.pointtoCount() != M_command_counter[PlayerCommand::POINTTO] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost pointto? at " << M_last_action_time
                   << " sense=" << sense.pointtoCount()
                   << " internal=" << M_command_counter[PlayerCommand::POINTTO]
@@ -354,7 +356,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 
     if ( sense.attentiontoCount() != M_command_counter[PlayerCommand::ATTENTIONTO] )
     {
-        std::cout << M_world.self().unum()
+        std::cout << M_agent.world().self().unum()
                   << ": lost attentionto? at " << M_last_action_time
                   << " sense=" << sense.attentiontoCount()
                   << " internal=" << M_command_counter[PlayerCommand::ATTENTIONTO]
@@ -374,14 +376,14 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
 std::ostream &
 ActionEffector::makeCommand( std::ostream & to )
 {
-    M_last_action_time = M_world.time();
+    M_last_action_time = M_agent.world().time();
 
     if ( M_command_body )
     {
         M_last_body_command_type = M_command_body->type();
         if ( M_last_body_command_type == PlayerCommand::CATCH )
         {
-            M_catch_time = M_world.time();
+            M_catch_time = M_agent.world().time();
         }
         M_command_body->toStr( to );
         incCommandCount( M_command_body->type() );
@@ -390,13 +392,13 @@ ActionEffector::makeCommand( std::ostream & to )
     }
     else
     {
-        if ( ! M_world.self().isFreezed() )
+        if ( ! M_agent.world().self().isFreezed() )
         {
             dlog.addText( Logger::SYSTEM,
                           "  WARNING. no body command." );
-            std::cerr << M_world.teamName() << ' '
-                      << M_world.self().unum()<< ": "
-                      << M_world.time()
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum()<< ": "
+                      << M_agent.world().time()
                       << "  WARNING. no body command." << std::endl;
             // register dummy command
             PlayerTurnCommand turn( 0 );
@@ -444,14 +446,15 @@ ActionEffector::makeCommand( std::ostream & to )
         done_com.toStr( to );
     }
 
+    makeSayCommand();
     if ( M_command_say )
     {
         M_command_say->toStr( to );
         incCommandCount( PlayerCommand::SAY );
         delete M_command_say;
         M_command_say = static_cast< PlayerSayCommand * >( 0 );
-        M_say_message = "";
     }
+    clearSayMessages();
 
     return to;
 }
@@ -471,9 +474,9 @@ ActionEffector::setKick( const double & power,
     double command_power = power;
     if ( command_power > ServerParam::i().maxPower() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " kick power is over max. com=" << command_power
                   << " > sparam=" << ServerParam::i().maxPower() << std::endl;
         dlog.addText( Logger::ACTION,
@@ -483,9 +486,9 @@ ActionEffector::setKick( const double & power,
     }
     if ( command_power < 0.0 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " kick power is negative " << command_power
                   << std::endl;
         dlog.addText( Logger::ACTION,
@@ -498,8 +501,8 @@ ActionEffector::setKick( const double & power,
 
     dlog.addText( Logger::ACTION,
                   "Effector::setKick. Power= %.1f  Dir= %.1f  KickRate= %.4f  Accel= %.2f",
-                  command_power, rel_dir.degree(), M_world.self().kickRate(),
-                  M_world.self().kickRate() * command_power );
+                  command_power, rel_dir.degree(), M_agent.world().self().kickRate(),
+                  M_agent.world().self().kickRate() * command_power );
 
     //////////////////////////////////////////////////
     // create command object
@@ -511,8 +514,8 @@ ActionEffector::setKick( const double & power,
     M_command_body = new PlayerKickCommand( command_power, rel_dir.degree() );
 
     // set estimated action effect
-    M_kick_accel.setPolar( command_power * M_world.self().kickRate(),
-                           M_world.self().body() + rel_dir );
+    M_kick_accel.setPolar( command_power * M_agent.world().self().kickRate(),
+                           M_agent.world().self().body() + rel_dir );
 
     // rcssserver/src/player.cc
     //  add noise to kick
@@ -527,7 +530,7 @@ ActionEffector::setKick( const double & power,
     // M_kick_accel_error *= M_kick_accel.r();
     // M_kick_accel_error.add(maxrnd, maxrnd);
 
-    double maxrnd = ( M_world.self().playerType().kickRand()
+    double maxrnd = ( M_agent.world().self().playerType().kickRand()
                       * command_power
                       / ServerParam::i().maxPower() );
     M_kick_accel_error.assign( maxrnd, maxrnd );
@@ -626,9 +629,9 @@ ActionEffector::setDash( const double & power )
     {
         dlog.addText( Logger::ACTION,
                       "dash power is too high" );
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " dash power is too high: " << power << std::endl;
         command_power = ServerParam::i().maxPower();
     }
@@ -636,14 +639,14 @@ ActionEffector::setDash( const double & power )
     {
         dlog.addText( Logger::ACTION,
                       "dash power is too low" );
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " dash power is too low;: " << power << std::endl;
         command_power = ServerParam::i().minPower();
     }
 
-    command_power = conserve_dash_power( M_world, command_power );
+    command_power = conserve_dash_power( M_agent.world(), command_power );
 
     command_power = rint( command_power );
 
@@ -658,9 +661,9 @@ ActionEffector::setDash( const double & power )
 
     // set estimated command effect
     const double accel_mag
-        = ( M_world.self().effort()
+        = ( M_agent.world().self().effort()
             * command_power
-            * M_world.self().playerType().dashPowerRate() );
+            * M_agent.world().self().playerType().dashPowerRate() );
     /*
      * accel_mag = std::min( effort * command_power * mydprate,
      *                       ServerParam::playerAccelMax() );
@@ -668,12 +671,12 @@ ActionEffector::setDash( const double & power )
 
     M_dash_power = command_power;
     M_dash_accel.setPolar( accel_mag,
-                           M_world.self().body() );
+                           M_agent.world().self().body() );
 
 #if 0
     double mincos, maxcos, minsin, maxsin;
-    M_world.self().body().getCosMinMax( M_world.self().faceError(), &mincos, &maxcos );
-    M_world.self().body().getSinMinMax( M_world.self().faceError(), &minsin, &maxsin );
+    M_agent.world().self().body().getCosMinMax( M_agent.world().self().faceError(), &mincos, &maxcos );
+    M_agent.world().self().body().getSinMinMax( M_agent.world().self().faceError(), &minsin, &maxsin );
 
     // set only dir info, radius is 1.0
     M_dash_accel_error.assign( ( maxcos - mincos ) * 0.5,
@@ -711,18 +714,18 @@ ActionEffector::setTurn( const AngleDeg & moment )
                   "register turn. moment = %.1f",
                   moment.degree() );
 
-    const double my_speed = M_world.self().vel().r();
+    const double my_speed = M_agent.world().self().vel().r();
     double command_moment = moment.degree();
     // required turn param
     command_moment
         *= ( 1.0
              + ( my_speed
-                 * M_world.self().playerType().inertiaMoment() ) );
+                 * M_agent.world().self().playerType().inertiaMoment() ) );
 
     dlog.addText( Logger::ACTION,
                   "register turn. moment= %.1f, cmd_param= %.1f, my_inertia= %.1f",
                   moment.degree(), command_moment,
-                  M_world.self().playerType().inertiaMoment() );
+                  M_agent.world().self().playerType().inertiaMoment() );
 
     // check parameter range
     if ( command_moment > ServerParam::i().maxMoment() )
@@ -767,7 +770,7 @@ ActionEffector::setTurn( const AngleDeg & moment )
     */
     // convert to real moment
     M_turn_actual = command_moment
-        / ( 1.0 + my_speed * M_world.self().playerType().inertiaMoment() );
+        / ( 1.0 + my_speed * M_agent.world().self().playerType().inertiaMoment() );
     M_turn_error = std::fabs( ServerParam::i().playerRand()
                               * M_turn_actual );
 
@@ -797,9 +800,9 @@ ActionEffector::setMove( const double & x,
     if ( std::fabs( command_y ) > ServerParam::i().pitchHalfWidth()
          || std::fabs( command_x ) > ServerParam::i().pitchHalfLength() )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " Must move to a place in the pitch ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
@@ -818,9 +821,9 @@ ActionEffector::setMove( const double & x,
     if ( ServerParam::i().kickoffOffside()
          && command_x > 0.0 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << " Must move to a place in our half ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
@@ -831,14 +834,14 @@ ActionEffector::setMove( const double & x,
 
     // when goalie catch mode,
     // move point must be in our penalty area
-    if ( M_world.gameMode().type() == GameMode::GoalieCatch_
-         && M_world.gameMode().side() == M_world.ourSide() )
+    if ( M_agent.world().gameMode().type() == GameMode::GoalieCatch_
+         && M_agent.world().gameMode().side() == M_agent.world().ourSide() )
     {
         if ( command_x < -ServerParam::i().pitchHalfLength() + 1.0 )
         {
-            std::cerr << M_world.teamName() << ' '
-                      << M_world.self().unum() << ": "
-                      << M_world.time()
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
                       << " Must move to a place within penalty area(1) ("
                       << command_x << ", " << command_y << ")" << std::endl;
             dlog.addText( Logger::ACTION,
@@ -848,9 +851,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_x > - ServerParam::i().ourPenaltyAreaLine() - 1.0 )
         {
-            std::cerr << M_world.teamName() << ' '
-                      << M_world.self().unum() << ": "
-                      << M_world.time()
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
                       << " Must move to a place within penalty area(2) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -860,9 +863,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_y > ServerParam::i().penaltyAreaHalfWidth() - 1.0 )
         {
-            std::cerr << M_world.teamName() << ' '
-                      << M_world.self().unum() << ": "
-                      << M_world.time()
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
                       << " Must move to a place within penalty area(3) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -872,9 +875,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_y < - ServerParam::i().penaltyAreaHalfWidth() + 1.0 )
         {
-            std::cerr << M_world.teamName() << ' '
-                      << M_world.self().unum() << ": "
-                      << M_world.time()
+            std::cerr << M_agent.world().teamName() << ' '
+                      << M_agent.world().self().unum() << ": "
+                      << M_agent.world().time()
                       << " Must move to a place within penalty area(4) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -913,7 +916,7 @@ ActionEffector::setCatch()
                                ServerParam::i().catchAreaLength() );
 
     // relative angle
-    AngleDeg catch_angle = M_world.ball().angleFromSelf() - M_world.self().body();
+    AngleDeg catch_angle = M_agent.world().ball().angleFromSelf() - M_agent.world().self().body();
     dlog.addText( Logger::ACTION,
                   "Effector::setCatch. ball_rel_angle= %.1f  diagonal_angle= %.1f",
                   catch_angle.degree(), diagonal_angle );
@@ -946,9 +949,9 @@ ActionEffector::setTackle( const double & power )
 
     if ( command_power > ServerParam::i().maxPower() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << "tackle power overflow. com=" << command_power
                   << " > sparam=" << ServerParam::i().maxPower() << std::endl;
         dlog.addText( Logger::ACTION,
@@ -958,9 +961,9 @@ ActionEffector::setTackle( const double & power )
     }
     if ( command_power < ServerParam::i().minPower() - 0.01 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << "tackle power underflow " << command_power << std::endl;
         dlog.addText( Logger::ACTION,
                       "Effector::setKick. power underflow. %.1f",
@@ -982,8 +985,8 @@ ActionEffector::setTackle( const double & power )
     // set estimated command effect
     M_tackle_power = command_power;
     M_tackle_dir = ( command_power > 0.0
-                     ? M_world.self().body().degree()
-                     : ( M_world.self().body() + 180.0 ).degree() );
+                     ? M_agent.world().self().body().degree()
+                     : ( M_agent.world().self().body() + 180.0 ).degree() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1002,9 +1005,9 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     // adjust for neck moment range
     if ( command_moment > ServerParam::i().maxNeckMoment() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << "  doTurnNeck. over max moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
@@ -1014,9 +1017,9 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     }
     if ( command_moment < ServerParam::i().minNeckMoment() - 0.01 )
     {
-        std::cerr << M_world.teamName() << ' '
-                  << M_world.self().unum() << ": "
-                  << M_world.time()
+        std::cerr << M_agent.world().teamName() << ' '
+                  << M_agent.world().self().unum() << ": "
+                  << M_agent.world().time()
                   << "  doTurnNeck. under min moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
@@ -1028,13 +1031,13 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     command_moment = rint( command_moment );
 
     // adjust for neck angle range
-    AngleDeg next_neck_angle = M_world.self().neck();
+    AngleDeg next_neck_angle = M_agent.world().self().neck();
     next_neck_angle += command_moment;
 
     if ( next_neck_angle.degree() > ServerParam::i().maxNeckAngle() )
     {
         command_moment = rint( ServerParam::i().maxNeckAngle()
-                               - M_world.self().neck().degree() );
+                               - M_agent.world().self().neck().degree() );
         dlog.addText( Logger::ACTION,
                       "Effector::setTurnNeck. next_neck= %.1f. over max. new-moment= %.1f",
                       next_neck_angle.degree(), command_moment );
@@ -1043,7 +1046,7 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     if ( next_neck_angle.degree() < ServerParam::i().minNeckAngle() )
     {
         command_moment = rint( ServerParam::i().minNeckAngle()
-                               - M_world.self().neck().degree() );
+                               - M_agent.world().self().neck().degree() );
         dlog.addText( Logger::ACTION,
                       "Effector::setTurnNeck. next_neck= %.1f. under min. new-momment= %.1f",
                       next_neck_angle.degree(), command_moment );
@@ -1087,24 +1090,76 @@ ActionEffector::setChangeView( const ViewWidth & width )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+// void
+// ActionEffector::setSay( const std::string & msg,
+//                         const double & version )
+// {
+//     dlog.addText( Logger::ACTION,
+//                   "register say. [%s]",
+//                   msg.c_str() );
+
+//     M_protocl_version = version;
+
+//     //////////////////////////////////////////////////
+//     // create command object
+//     if ( M_command_say )
+//     {
+//         //delete M_command_say;
+//         //M_command_say = static_cast< PlayerSayCommand * >( 0 );
+//         M_command_say->assign( msg );
+//     }
+//     else
+//     {
+//         M_command_say = new PlayerSayCommand( msg, version );
+//     }
+
+//     M_say_message = msg;
+// }
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
-ActionEffector::setSay( const std::string & msg,
-                        const double & version )
+ActionEffector::addSayMessage( const SayMessage * message )
 {
     dlog.addText( Logger::ACTION,
-                  "register say. [%s]",
-                  msg.c_str() );
+                  "add new say message." );
 
-    //////////////////////////////////////////////////
-    // create command object
-    if ( M_command_say )
+    M_say_messages.push_back( message );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+ActionEffector::removeSayMessage( const char header )
+{
+    dlog.addText( Logger::ACTION,
+                  __FILE__": remove say message. header = %c", header );
+
+    bool removed = false;
+
+    std::vector< const SayMessage * >::iterator it = M_say_messages.begin();
+
+    while ( it != M_say_messages.end() )
     {
-        delete M_command_say;
-        M_command_say = static_cast< PlayerSayCommand * >( 0 );
+        if ( (*it)->header() == header )
+        {
+            it = M_say_messages.erase( it );
+            removed = true;
+            dlog.addText( Logger::ACTION,
+                          __FILE__": removed" );
+        }
+        else
+        {
+            ++it;
+        }
     }
-    M_say_message = msg;
-    M_command_say = new PlayerSayCommand( msg, version );
+
+    return removed;
 }
 
 /*-------------------------------------------------------------------*/
@@ -1120,7 +1175,7 @@ ActionEffector::setPointto( const double & x,
                   x, y );
 
     Vector2D target_pos( x, y );
-    Vector2D target_rel = target_pos - M_world.self().pos();
+    Vector2D target_rel = target_pos - M_agent.world().self().pos();
 
     //////////////////////////////////////////////////
     // create command object
@@ -1180,7 +1235,7 @@ ActionEffector::setAttentionto( const SideID side,
     }
 
     M_command_attentionto
-        = new PlayerAttentiontoCommand( ( M_world.ourSide() == side
+        = new PlayerAttentiontoCommand( ( M_agent.world().ourSide() == side
                                           ? PlayerAttentiontoCommand::OUR
                                           : PlayerAttentiontoCommand::OPP ),
                                         unum );
@@ -1206,7 +1261,82 @@ ActionEffector::setAttentiontoOff()
     M_command_attentionto = new PlayerAttentiontoCommand();
 }
 
+/*-------------------------------------------------------------------*/
+/*!
 
+*/
+int
+ActionEffector::getSayMessageLength() const
+{
+    int len = 0;
+
+    const std::vector< const SayMessage * >::const_iterator end = M_say_messages.end();
+    for ( std::vector< const SayMessage * >::const_iterator it = M_say_messages.begin();
+          it != end;
+          ++it )
+    {
+        len += (*it)->length();
+    }
+
+    return len;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+ActionEffector::makeSayCommand()
+{
+    if ( M_command_say )
+    {
+        delete M_command_say;
+        M_command_say = static_cast< PlayerSayCommand * >( 0 );
+    }
+
+    M_say_message.erase();
+
+    // std::sort( M_say_messages.begin(), M_say_messages.end(),
+    //            SayMessagePtrSorter() );
+
+    const std::vector< const SayMessage * >::const_iterator end = M_say_messages.end();
+    for ( std::vector< const SayMessage * >::const_iterator it = M_say_messages.begin();
+          it != end;
+          ++it )
+    {
+        (*it)->toStr( M_say_message );
+    }
+
+    if ( M_say_message.empty() )
+    {
+        return;
+    }
+
+    M_command_say = new PlayerSayCommand( M_say_message,
+                                          M_agent.config().version() );
+
+    dlog.addText( rcsc::Logger::ACTION,
+                  __FILE__": say message [%s]",
+                  M_say_message.c_str() );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+ActionEffector::clearSayMessages()
+{
+    const std::vector< const SayMessage * >::const_iterator end = M_say_messages.end();
+    for ( std::vector< const SayMessage * >::const_iterator it = M_say_messages.begin();
+          it != end;
+          ++it )
+    {
+        delete *it;
+    }
+
+    M_say_messages.clear();
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -1217,7 +1347,7 @@ ActionEffector::setAttentiontoOff()
 AngleDeg
 ActionEffector::queuedNextMyBody() const
 {
-    AngleDeg next_angle = M_world.self().body();
+    AngleDeg next_angle = M_agent.world().self().body();
     if ( M_command_body
          && M_command_body->type() == PlayerCommand::TURN )
     {
@@ -1226,7 +1356,7 @@ ActionEffector::queuedNextMyBody() const
         next_angle += moment;
         dlog.addText( Logger::SYSTEM,
                       "effector.queuedNextMyBody. cur_body= %f + moment= %f ->  %f",
-                      M_world.self().body().degree(), moment, next_angle.degree() );
+                      M_agent.world().self().body().degree(), moment, next_angle.degree() );
     }
 
     return next_angle;
@@ -1234,12 +1364,12 @@ ActionEffector::queuedNextMyBody() const
 
 /*-------------------------------------------------------------------*/
 /*!
-  estimate my next pos
+
 */
 Vector2D
 ActionEffector::queuedNextMyPos() const
 {
-    Vector2D vel = M_world.self().vel();
+    Vector2D vel = M_agent.world().self().vel();
     if ( M_command_body
          && M_command_body->type() == PlayerCommand::DASH )
     {
@@ -1248,13 +1378,13 @@ ActionEffector::queuedNextMyPos() const
         vel += accel;
 
         double tmp = vel.r();
-        if ( tmp > M_world.self().playerType().playerSpeedMax() )
+        if ( tmp > M_agent.world().self().playerType().playerSpeedMax() )
         {
-            vel *= M_world.self().playerType().playerSpeedMax() / tmp;
+            vel *= M_agent.world().self().playerType().playerSpeedMax() / tmp;
         }
     }
 
-    return M_world.self().pos() + vel;
+    return M_agent.world().self().pos() + vel;
 }
 
 /*-------------------------------------------------------------------*/
@@ -1264,7 +1394,7 @@ ActionEffector::queuedNextMyPos() const
 bool
 ActionEffector::queuedNextBallKickable() const
 {
-    if ( M_world.ball().rposCount() >= 3 )
+    if ( M_agent.world().ball().rposCount() >= 3 )
     {
         return false;
     }
@@ -1272,26 +1402,26 @@ ActionEffector::queuedNextBallKickable() const
     Vector2D my_next = queuedNextMyPos();
     Vector2D ball_next = queuedNextBallPos();
 
-    return my_next.dist( ball_next ) < M_world.self().kickableArea() - 0.06;
+    return my_next.dist( ball_next ) < M_agent.world().self().kickableArea() - 0.06;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  estimate next ball pos
+
 */
 Vector2D
 ActionEffector::queuedNextBallPos() const
 {
-    if ( ! M_world.ball().posValid() )
+    if ( ! M_agent.world().ball().posValid() )
     {
         return Vector2D::INVALIDATED;
     }
 
     Vector2D vel( 0.0, 0.0 ), accel( 0.0, 0.0 );
 
-    if ( M_world.ball().velValid() )
+    if ( M_agent.world().ball().velValid() )
     {
-        vel = M_world.ball().vel();
+        vel = M_agent.world().ball().vel();
     }
 
     if ( M_command_body
@@ -1302,21 +1432,21 @@ ActionEffector::queuedNextBallPos() const
 
     vel += accel;
 
-    return M_world.ball().pos() + vel;
+    return M_agent.world().ball().pos() + vel;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  estimate next ball vel
+
 */
 Vector2D
 ActionEffector::queuedNextBallVel() const
 {
     Vector2D vel( 0.0, 0.0 ), accel( 0.0, 0.0 );
 
-    if ( M_world.ball().velValid() )
+    if ( M_agent.world().ball().velValid() )
     {
-        vel = M_world.ball().vel();
+        vel = M_agent.world().ball().vel();
     }
 
     if ( M_command_body
@@ -1332,7 +1462,7 @@ ActionEffector::queuedNextBallVel() const
 
 /*-------------------------------------------------------------------*/
 /*!
-  calc the relative angle from my predict pos to target
+
 */
 AngleDeg
 ActionEffector::queuedNextAngleFromBody( const Vector2D & target ) const
@@ -1344,7 +1474,7 @@ ActionEffector::queuedNextAngleFromBody( const Vector2D & target ) const
 
 /*-------------------------------------------------------------------*/
 /*!
-  get the view width when next see arrival
+
 */
 ViewWidth
 ActionEffector::queuedNextViewWidth() const
@@ -1354,7 +1484,7 @@ ActionEffector::queuedNextViewWidth() const
         return M_command_change_view->width();
     }
 
-    return M_world.self().viewWidth();
+    return M_agent.world().self().viewWidth();
 }
 
 }
