@@ -41,12 +41,14 @@
 
 #include <iostream>
 #include <string>
+#include <vector>
 
 namespace rcsc {
 
-class ServerParam;
 class BodySensor;
-class WorldModel;
+class PlayerAgent;
+class SayMessage;
+class ServerParam;
 
 /*!
   \class ActionEffector
@@ -54,8 +56,8 @@ class WorldModel;
 */
 class ActionEffector {
 private:
-    //! const reference to the WorldModel instance
-    const WorldModel & M_world;
+    //! const reference to the PlayerAgent instance
+    const PlayerAgent & M_agent;
 
     //! pointer of body action for dynamic allocation
     PlayerBodyCommand * M_command_body;
@@ -112,7 +114,9 @@ private:
     double M_turn_neck_moment; //!< last turn_neck moment
 
     // say effect
+    double M_protocol_version; //!< client protocl version
     std::string M_say_message; //!< last said message string
+    std::vector< const SayMessage * > M_say_messages;
 
     // pointto effect
     Vector2D M_pointto_pos;  //!< last pointto coordinates
@@ -126,10 +130,10 @@ private:
 public:
     /*!
       \brief init member variables
-      \param world const reference to the WorldModel instance
+      \param agent const reference to the PlayerAgent instance
     */
     explicit
-    ActionEffector( const WorldModel & world );
+    ActionEffector( const PlayerAgent & agent );
 
     /*!
       \brief destruct dynamic allocated command objects
@@ -137,7 +141,7 @@ public:
     ~ActionEffector();
 
     /*!
-      \brief reset all command effect. called just before action decition
+      \brief reset all command effect. called just before action decision
     */
     void reset();
 
@@ -306,8 +310,21 @@ public:
       \param msg say message string
       \param version client version
     */
-    void setSay( const std::string & msg,
-                 const double & version );
+    //void setSay( const std::string & msg,
+    //const double & version );
+
+    /*!
+      \brief add new say message
+      \param message say message object. this must be a dynamically allocated object.
+     */
+    void addSayMessage( const SayMessage * message );
+
+    /*!
+      \brief remove the registered say message if exist
+      \param header message header character
+      \return true if removed
+     */
+    bool removeSayMessage( const char header );
 
     /*!
       \brief create pointto command and its effect with pointto parameter
@@ -472,6 +489,22 @@ public:
           return M_say_message;
       }
 
+    /*!
+      \brief get the total length of current reserved messages
+      \return the total length of current reserved messages
+     */
+    int getSayMessageLength() const;
+
+    /*!
+      \brief get the reserved say messages
+      \return const reference to the say message builder container
+     */
+    const
+    std::vector< const SayMessage * > & sayMessageCont() const
+      {
+          return M_say_messages;
+      }
+
     //////////////////////////////////////////
     /*!
       \brief get pointto action effect
@@ -527,6 +560,19 @@ public:
       \return queued view width
     */
     ViewWidth queuedNextViewWidth() const;
+
+private:
+
+    /*!
+      \brief create say command object using the registered say message objects
+     */
+    void makeSayCommand();
+
+    /*!
+      \brief erase all say message objects
+     */
+    void clearSayMessages();
+
 };
 
 }
