@@ -39,7 +39,7 @@
 #include "logger.h"
 #include "player_command.h"
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/game_mode.h>
 
 #include <iostream>
@@ -69,9 +69,7 @@ BallObject::BallObject()
     , M_vel_count( 1000 )
     , M_last_seen_pos( Vector2D::INVALIDATED )
     , M_lost_count( 0 )
-    , M_heard_time( 0, 0 )
-    , M_heard_pos( 0.0, 0.0 )
-    , M_heard_vel( 0.0, 0.0 )
+    , M_ghost_time( -1, 0 )
 {
 
 }
@@ -95,12 +93,13 @@ BallObject::set_count_thr( const int pos_thr,
 
 */
 void
-BallObject::forget()
+BallObject::setGhost( const GameTime & current )
 {
     M_dist_from_self = 1000.0;
     M_pos_count = 1000;
     M_rpos_count = 1000;
     M_lost_count = 0;
+    M_ghost_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -312,15 +311,18 @@ BallObject::updateWindEffect()
 */
 void
 BallObject::updateByCollision( const Vector2D & pos,
+                               const int pos_count,
                                const Vector2D & rpos,
-                               const Vector2D & vel )
+                               const int rpos_count,
+                               const Vector2D & vel,
+                               const int vel_count )
 {
     M_pos = pos;
-    ++M_pos_count;
+    M_pos_count = pos_count;
     M_rpos = rpos;
-    ++M_rpos_count;
+    M_rpos_count = rpos_count;
     M_vel = vel;
-    ++M_vel_count;
+    M_vel_count = vel_count;
 }
 
 /*-------------------------------------------------------------------*/
@@ -391,33 +393,42 @@ BallObject::updateAll( const Vector2D & pos,
 
 */
 void
-BallObject::setHeardInfo( const Vector2D & heard_pos,
+BallObject::updateByHear( const Vector2D & heard_pos,
                           const Vector2D & heard_vel,
-                          const GameTime & heard_time )
+                          const GameTime & current )
 {
-    M_heard_time = heard_time;
-    M_heard_pos = heard_pos;
-    M_heard_vel = heard_vel;
-}
+    if ( M_ghost_time == current )
+    {
+        if ( heard_pos.dist( M_pos ) < 3.0 )
+        {
+            dlog.addText( Logger::WORLD,
+                          __FILE__": Ball. updateByHeardInfo. ghost detected. heard_pos=(%.2f, %.2f)"
+                          " heard_vel=(%.2f, %.2f)",
+                          heard_pos.x, heard_pos.y,
+                          heard_vel.x, heard_vel.y );
+            return;
+        }
+    }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-BallObject::updateByHeardInfo( const GameTime & current )
-{
-    if ( posCount() >= 2
-         && current == M_heard_time )
+    if ( posCount() >= 2 )
     {
         dlog.addText( Logger::WORLD,
-                      "Ball. updateByHeardInfo. heard_pos=(%.2f, %.2f)"
+                      __FILE__": Ball. updateByHeardInfo. heard_pos=(%.2f, %.2f)"
                       " heard_vel=(%.2f, %.2f)",
-                      M_heard_pos.x, M_heard_pos.y,
-                      M_heard_vel.x, M_heard_vel.y );
-        M_pos = M_heard_pos;
+                      heard_pos.x, heard_pos.y,
+                      heard_vel.x, heard_vel.y );
+        M_pos = heard_pos;
         M_pos_count = 1;
-        M_vel = M_heard_vel;
+        M_vel = heard_vel;
+        M_vel_count = 1;
+    }
+    else if ( velCount() >= 2 )
+    {
+        dlog.addText( Logger::WORLD,
+                      __FILE__": Ball. updateByHeardInfo. vel only"
+                      " heard_vel=(%.2f, %.2f)",
+                      heard_vel.x, heard_vel.y );
+        M_vel = heard_vel;
         M_vel_count = 1;
     }
 }
