@@ -94,12 +94,8 @@ private:
     //! count from last see observation
     int M_lost_count;
 
-    //! time heared audio
-    GameTime M_heard_time;
-    //! heard global coordinate
-    Vector2D M_heard_pos;
-    //! heard global velocity
-    Vector2D M_heard_vel;
+    //! last ghost check time
+    GameTime M_ghost_time;
 
     //! not used
     BallObject( const BallObject & ball );
@@ -260,33 +256,13 @@ public:
       }
 
     /*!
-      \brief get heard time
+      \brief get ghost detected time
       \return game time value
      */
     const
-    GameTime & heardTime() const
+    GameTime & ghostTime() const
       {
-          return M_heard_time;
-      }
-
-    /*!
-      \brief get heard position
-      \return position value
-     */
-    const
-    Vector2D & heardPos() const
-      {
-          return M_heard_pos;
-      }
-
-    /*!
-      \brief get heard velocity
-      \return velocity value
-     */
-    const
-    Vector2D & heardVel() const
-      {
-          return M_heard_vel;
+          return M_ghost_time;
       }
 
     /*!
@@ -317,9 +293,10 @@ public:
       }
 
     /*!
-      \brief clear all confidence values
+      \brief clear all confidence values and set ghost time
+      \param current current game time
     */
-    void forget();
+    void setGhost( const GameTime & current );
 
     /*!
       \brief update status only with intenal info
@@ -352,8 +329,11 @@ public:
     /*!
       \brief apply collision effect
       \param pos new global position
+      \param pos_count accuracy counter
       \param rpos new relative position
+      \param rpos_count accuracy counter
       \param vel new velocity
+      \param vel_count accuracy counter
 
       Thie method is called when collision estimatation
     */
@@ -414,23 +394,17 @@ public:
                     const Vector2D & vel,
                     const Vector2D & vel_err );
 
-    /*!
-      \brief set heard ball info
-      \param heard_pos heard global position
-      \param heard_vel heard velocity
-      \param heard_time time when hear info is received
-    */
-    void setHeardInfo( const Vector2D & heard_pos,
-                       const Vector2D & heard_vel,
-                       const GameTime & heard_time );
-
     /*!
       \brief update ball status using heared info
+      \param heard_pos heard position
+      \param heard_vel heard velocity
       \param current current game time
 
       This method is called just before decision.
     */
-    void updateByHeardInfo( const GameTime & current );
+    void updateByHear( const Vector2D & heard_pos,
+                       const Vector2D & heard_vel,
+                       const GameTime & current );
 
     /*!
       \brief update self related info
