@@ -53,8 +53,8 @@ public:
     ViewWidth M_view_width; //!< sensed view width
     double M_stamina; //!< sensed stamina value
     double M_effort; //!< sensed effort value
-    double M_speed_mag; //! sensed speed magnitude. this is quantized by 0.01.
-    double M_speed_dir_relative; //! speed dir. this is relative to face angle.
+    double M_speed_mag; //!< sensed speed magnitude. this is quantized by 0.01.
+    double M_speed_dir_relative; //!< speed dir. this is relative to face angle.
     double M_neck_relative; //!< neck angle. this is relative to body angle
 
     int M_kick_count; //!< sensed command count
