@@ -34,6 +34,7 @@
 #endif
 
 #include "debug_client.h"
+#include "logger.h"
 #include "world_model.h"
 #include "player_object.h"
 
@@ -52,6 +53,7 @@ namespace rcsc {
 // accessible only from this file
 namespace {
 
+//! max send buffer size for debug client
 #define G_BUFFER_SIZE 8192*4
 
 //! global variable
@@ -84,7 +86,7 @@ public:
     void operator()( const PlayerObject & p )
       {
           M_os << " (";
-          if ( p.unum() != -1 )
+          if ( p.unum() != Unum_Unknown )
           {
               M_os << M_tag << ' ' << p.unum();
           }
@@ -100,7 +102,7 @@ public:
           M_os << ' ' << ROUND( p.pos().x, 0.01 )
                << ' ' << ROUND( p.pos().y, 0.01 );
 
-          if ( p.faceValid() )
+          if ( p.bodyValid() )
           {
               M_os << " (bd " << rint( p.body().degree() )
                    << ')';
@@ -205,15 +207,14 @@ public:
 DebugClient::DebugClient()
     : M_on( false )
     , M_connected( false )
+    , M_write_mode( false )
     , M_main_buffer( "" )
     , M_target_unum( 0 )
     , M_target_point( Vector2D::INVALIDATED )
     , M_message( "" )
-    , M_freeforms( "" )
 {
     M_main_buffer.reserve( 8192 );
     M_message.reserve( 8192 );
-    M_freeforms.reserve( 8192 * 2 );
     M_lines.reserve( 100 );
 }
 
@@ -257,34 +258,13 @@ DebugClient::connect( const std::string & hostname,
 
 */
 void
-DebugClient::open( const std::string & teamname,
-                   const int unum,
-                   const std::string & file_dir )
+DebugClient::setWriteMode( const bool on )
 {
-    std::ostringstream ostr;
-
-    if ( file_dir.empty() )
+    if ( on )
     {
-        ostr << "./";
+        M_on = true;
+        M_write_mode = true;
     }
-    else
-    {
-        ostr << file_dir;
-        if ( file_dir[file_dir.length() - 1] != '/' )
-        {
-            ostr << '/';
-        }
-    }
-    ostr << teamname << "-" << unum << ".slog";
-
-    M_file.open( ostr.str().c_str() );
-    if ( ! M_file.is_open() )
-    {
-        std::cout << "fail to open Soccer Viewer Log File ["
-                  << ostr.str() << "]" << std::endl;
-        return;
-    }
-    M_on = true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -303,8 +283,8 @@ DebugClient::output( const WorldModel & world )
             this->send();
         }
 
-        if ( world.time().stopped() == 0
-             && M_file.is_open() )
+        if ( M_write_mode
+             && world.time().stopped() == 0 )
         {
             this->write( world.time().cycle() );
         }
@@ -326,10 +306,9 @@ DebugClient::close()
         M_socket.reset();
     }
 
-    if ( M_file.is_open() )
-    {
-        M_file.close();
-    }
+    M_write_mode = false;
+
+    M_on = false;
 }
 
 /*-------------------------------------------------------------------*/
@@ -351,7 +330,7 @@ DebugClient::toStr( const WorldModel & world )
       BODY_DIRECTION FACE_DIRECTION  [(c "COMMENT")])
       where SIDE is 'l' or 'r'.
       where PLAYER_NUMBER is 1 to 11.
-      where POS_X and POS_Y is absolute coodinate.
+      where POS_X and POS_Y is absolute coordinate.
       where VEL_X and VEL_Y is absolute velocity.
       where BODY_DIRECTION is absolute player body direction (degree).
       where NECK_DIRECTION is body relative face direction (degree).
@@ -417,18 +396,10 @@ DebugClient::toStr( const WorldModel & world )
                    world.teammates().end(),
                    PlayerPrinter( ostr, 't' ) );
 
-    std::for_each( world.unknownTeammates().begin(),
-                   world.unknownTeammates().end(),
-                   PlayerPrinter( ostr, 't' ) );
-
     std::for_each( world.opponents().begin(),
                    world.opponents().end(),
                    PlayerPrinter( ostr, 'o' ) );
 
-    std::for_each( world.unknownOpponents().begin(),
-                   world.unknownOpponents().end(),
-                   PlayerPrinter( ostr, 'o' ) );
-
     std::for_each( world.unknownPlayers().begin(),
                    world.unknownPlayers().end(),
                    PlayerPrinter( ostr, 'u' ) );
@@ -496,12 +467,14 @@ DebugClient::send()
 void
 DebugClient::write( const long & cycle )
 {
-    if ( M_file.is_open() )
+    if ( dlog.isOpen() )
     {
-        M_file << "%% step " << cycle << "\n"
-               << "%% debug [" << M_main_buffer << "]\n"
-               << M_freeforms << '\n';
-        M_file.flush();
+        char buf[32];
+        std::snprintf( buf, 32, "%%%% step %ld\n", cycle );
+        dlog.print( buf );
+        dlog.print( "%% debug [" );
+        dlog.print( M_main_buffer.c_str() );
+        dlog.print( "]\n" );
     }
 }
 
@@ -520,7 +493,6 @@ DebugClient::clear()
     M_triangles.clear();
     M_rectangles.clear();
     M_circles.clear();
-    M_freeforms.erase();
 }
 
 /*-------------------------------------------------------------------*/
@@ -607,23 +579,4 @@ DebugClient::addCircle( const Circle2D & circle )
     }
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-DebugClient::addFreeform( char * msg, ... )
-{
-    if ( M_on )
-    {
-        va_list argp;
-        va_start( argp, msg );
-        vsnprintf( g_buffer, G_BUFFER_SIZE, msg, argp );
-        va_end( argp );
-
-        M_freeforms += g_buffer;
-        M_freeforms += '\n';
-    }
-}
-
 }
