@@ -198,8 +198,8 @@ InterceptTable::update()
 
 */
 void
-InterceptTable::updateTeammate( const int unum,
-                                const int cycle )
+InterceptTable::hearTeammate( const int unum,
+                              const int cycle )
 {
     if ( M_fastest_teammate
          && cycle >= M_teammate_reach_cycle )
@@ -214,7 +214,7 @@ InterceptTable::updateTeammate( const int unum,
         M_teammate_reach_cycle = cycle;
 
         dlog.addText( Logger::INTERCEPT,
-                      "<-----By Heard: Intercept Teammate  fastest reach step = %d."
+                      "<----- Hear Intercept Teammate  fastest reach step = %d."
                       " teammate %d (%.1f %.1f)",
                       M_teammate_reach_cycle,
                       M_fastest_teammate->unum(),
@@ -226,6 +226,54 @@ InterceptTable::updateTeammate( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+InterceptTable::hearOpponent( const int unum,
+                              const int cycle )
+{
+    if ( M_fastest_opponent )
+    {
+        if ( cycle >= M_opponent_reach_cycle )
+        {
+            dlog.addText( Logger::INTERCEPT,
+                          "<----- Hear Intercept Opponent. no update."
+                          " exist faster reach cycle %d >= %d",
+                          cycle, M_opponent_reach_cycle );
+            return;
+        }
+
+        if ( M_fastest_opponent->unum() == unum
+             && M_fastest_opponent->posCount() == 0 )
+        {
+            dlog.addText( Logger::INTERCEPT,
+                          "<----- Hear Intercept Opponent . no update."
+                          " opponent %d (%.1f %.1f) is seen",
+                          M_fastest_opponent->unum(),
+                          M_fastest_opponent->pos().x,
+                          M_fastest_opponent->pos().y );
+            return;
+        }
+    }
+
+    const PlayerObject * p = M_world.getOpponent( unum );
+    if ( p )
+    {
+        M_fastest_opponent = p;
+        M_opponent_reach_cycle = cycle;
+
+        dlog.addText( Logger::INTERCEPT,
+                      "<----- Hear Intercept Opponent  fastest reach step = %d."
+                      " opponent %d (%.1f %.1f)",
+                      M_opponent_reach_cycle,
+                      M_fastest_opponent->unum(),
+                      M_fastest_opponent->pos().x,
+                      M_fastest_opponent->pos().y );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 InterceptTable::createBallCache()
@@ -332,12 +380,30 @@ InterceptTable::predictSelf()
 void
 InterceptTable::predictTeammate()
 {
+    const PlayerPtrCont & teammates = M_world.teammatesFromBall();
+    const PlayerPtrCont::const_iterator t_end = teammates.end();
+
     if ( M_world.existKickableTeammate() )
     {
         dlog.addText( Logger::INTERCEPT,
                       "Intercept Teammate. exist kickable teammate. no estimation loop!" );
         M_teammate_reach_cycle = 0;
-        M_fastest_teammate = M_world.teammatesFromBall().front();
+
+        for ( PlayerPtrCont::const_iterator t = teammates.begin();
+              t != t_end;
+              ++t )
+        {
+            if ( (*t)->isGhost()
+                   || (*t)->posCount() > M_world.ball().posCount() + 1 )
+              {
+                  continue;
+              }
+
+              if ( (*t)->isKickable() )
+              {
+                  M_fastest_teammate = *t;
+              }
+        }
         return;
     }
 
@@ -345,10 +411,8 @@ InterceptTable::predictTeammate()
 
     PlayerIntercept predictor( M_world, M_ball_pos_cache );
 
-
-    const PlayerPtrCont & teammates = M_world.teammatesFromBall();
     for ( PlayerPtrCont::const_iterator it = teammates.begin();
-          it != teammates.end();
+          it != t_end;
           ++it )
     {
         if ( (*it)->posCount() >= 10 )
@@ -406,12 +470,31 @@ InterceptTable::predictTeammate()
 void
 InterceptTable::predictOpponent()
 {
+    const PlayerPtrCont & opponents = M_world.opponentsFromBall();
+    const PlayerPtrCont::const_iterator o_end = opponents.end();
+
     if ( M_world.existKickableOpponent() )
     {
         dlog.addText( Logger::INTERCEPT,
                       "Intercept Opponent. exist kickable opponent. no estimation loop!" );
         M_opponent_reach_cycle = 0;
-        M_fastest_opponent = M_world.opponentsFromBall().front();
+
+        for ( PlayerPtrCont::const_iterator o = opponents.begin();
+              o != o_end;
+              ++o )
+        {
+            if ( (*o)->isGhost()
+                   || (*o)->posCount() > M_world.ball().posCount() + 1 )
+              {
+                  continue;
+              }
+
+              if ( (*o)->isKickable() )
+              {
+                  M_fastest_opponent = *o;
+              }
+        }
+
         return;
     }
 
@@ -419,9 +502,8 @@ InterceptTable::predictOpponent()
 
     PlayerIntercept predictor( M_world, M_ball_pos_cache );
 
-    const PlayerPtrCont & opponents = M_world.opponentsFromBall();
     for ( PlayerPtrCont::const_iterator it = opponents.begin();
-          it != opponents.end();
+          it != o_end;
           ++it )
     {
         if ( (*it)->posCount() >= 10 )
