@@ -47,6 +47,7 @@ using std::max;
 
 namespace rcsc {
 
+//! type of maerker map container
 typedef std::map< MarkerID, Vector2D > MarkerMap;
 
 /*!
@@ -670,8 +671,8 @@ LocalizeImpl::generateParticles( const VisualSensor::MarkerT & marker,
         base_angle += dir_inc;
     }
     dlog.addText( Logger::WORLD,
-                  "localizer.generateParticles. generate %d,  dir_inc= %f,  marker_dist= %f, "
-                  "  dist_range= %f,  circum= %f,  dir_divs= %f",
+                  "localizer.generateParticles. generate %d, dir_inc= %.1f, marker_dist= %.2f, "
+                  "  dist_range= %.1f,  circum= %.2f,  dir_divs= %.1f",
                   M_particles.size(), dir_inc, ave_dist,
                   dist_range, circum, dir_divs );
 
