@@ -40,9 +40,13 @@
 #include "audio_sensor.h"
 #include "fullstate_sensor.h"
 
+#include "audio_memory.h"
 #include "debug_client.h"
+#include "freeform_parser.h"
 #include "logger.h"
 #include "player_command.h"
+#include "say_message_builder.h"
+#include "say_message_parser.h"
 #include "soccer_action.h"
 #include "soccer_intention.h"
 
@@ -373,9 +377,31 @@ PlayerAgent::PlayerAgent()
     , M_impl( new PlayerAgentImpl( *this ) )
     , M_debug_client()
     , M_worldmodel()
-    , M_effector( M_worldmodel )
+    , M_effector( *this )
 {
     // std::cerr << "construct player" << std::endl;
+
+    typedef boost::shared_ptr< SayMessageParser > SMP;
+    typedef boost::shared_ptr< FreeformParser > FP;
+
+    boost::shared_ptr< AudioMemory > audio_memory( new AudioMemory );
+
+    M_worldmodel.setAudioMemory( audio_memory );
+
+    addSayMessageParser( SMP( new BallMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new PassMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new InterceptMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new GoalieMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new OffsideLineMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new DefenseLineMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new WaitRequestMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new PassRequestMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new DribbleMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new BallGoalieMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new OnePlayerMessageParser( audio_memory ) ) );
+    addSayMessageParser( SMP( new BallPlayerMessageParser( audio_memory ) ) );
+
+    M_impl->audio_.addParser( FP( new FreeformParser( M_worldmodel ) ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -503,12 +529,13 @@ PlayerAgent::initImpl( CmdLineParser & cmd_parser )
     // analyze command line for player options.
     cmd_parser.parse( player_param_map );
 
-    //if ( ! cmd_parser.invalidOptions().empty() )
-    //{
-    //    std::cerr << "player: ***WARNING*** detected invalid options: ";
-    //    cmd_parser.printError( std::cerr );
-    //    std::cerr << std::endl;
-    //}
+    if ( config().version() < 8.0
+         || 12.0 <= config().version() )
+    {
+        std::cerr << "Unsupported client version: " << config().version()
+                  << std::endl;
+        return false;
+    }
 
     setDebugFlags();
 
@@ -707,6 +734,27 @@ PlayerAgent::handleExit()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+PlayerAgent::addSayMessageParser( boost::shared_ptr< SayMessageParser > parser )
+{
+    M_impl->audio_.addParser( parser );
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+PlayerAgent::removeSayMessageParser( const char header )
+{
+    M_impl->audio_.removeSayMessageParser( header );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerAgent::finalize()
@@ -801,19 +849,19 @@ PlayerAgent::setDebugFlags()
         return;
     }
 
-    dlog.setLogFlag( this, Logger::SYSTEM, config().debugSystem() );
-    dlog.setLogFlag( this, Logger::SENSOR, config().debugSensor() );
-    dlog.setLogFlag( this, Logger::WORLD, config().debugWorld() );
-    dlog.setLogFlag( this, Logger::ACTION, config().debugAction() );
-    dlog.setLogFlag( this, Logger::INTERCEPT, config().debugIntercept() );
-    dlog.setLogFlag( this, Logger::KICK, config().debugKick() );
-    dlog.setLogFlag( this, Logger::DRIBBLE, config().debugDribble() );
-    dlog.setLogFlag( this, Logger::PASS, config().debugPass() );
-    dlog.setLogFlag( this, Logger::CROSS, config().debugCross() );
-    dlog.setLogFlag( this, Logger::SHOOT, config().debugShoot() );
-    dlog.setLogFlag( this, Logger::CLEAR, config().debugClear() );
-    dlog.setLogFlag( this, Logger::TEAM, config().debugTeam() );
-    dlog.setLogFlag( this, Logger::ROLE, config().debugRole() );
+    dlog.setLogFlag( &world().time(), Logger::SYSTEM, config().debugSystem() );
+    dlog.setLogFlag( &world().time(), Logger::SENSOR, config().debugSensor() );
+    dlog.setLogFlag( &world().time(), Logger::WORLD, config().debugWorld() );
+    dlog.setLogFlag( &world().time(), Logger::ACTION, config().debugAction() );
+    dlog.setLogFlag( &world().time(), Logger::INTERCEPT, config().debugIntercept() );
+    dlog.setLogFlag( &world().time(), Logger::KICK, config().debugKick() );
+    dlog.setLogFlag( &world().time(), Logger::DRIBBLE, config().debugDribble() );
+    dlog.setLogFlag( &world().time(), Logger::PASS, config().debugPass() );
+    dlog.setLogFlag( &world().time(), Logger::CROSS, config().debugCross() );
+    dlog.setLogFlag( &world().time(), Logger::SHOOT, config().debugShoot() );
+    dlog.setLogFlag( &world().time(), Logger::CLEAR, config().debugClear() );
+    dlog.setLogFlag( &world().time(), Logger::TEAM, config().debugTeam() );
+    dlog.setLogFlag( &world().time(), Logger::ROLE, config().debugRole() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1219,61 +1267,7 @@ PlayerAgent::analyzeHearPlayer(  const char * msg )
     }
 
     // parse message
-    M_impl->audio_.parsePlayer( msg, M_impl->current_time_ );
-
-    //----------------------------------------------
-    // set new info to world model
-    if ( M_impl->audio_.playerMessage().time_ == M_impl->current_time_ )
-    {
-        const std::vector< PlayerMessage::Info >::const_iterator end
-            = M_impl->audio_.playerMessage().info_.end();
-        for ( std::vector< PlayerMessage::Info >::const_iterator it
-                  = M_impl->audio_.playerMessage().info_.begin();
-              it != end;
-              ++it )
-        {
-            switch ( it->type_ ) {
-            case PlayerMessage::BALL_INFO:
-                M_worldmodel.setHeardBallInfo( M_impl->audio_.ballPos(),
-                                               M_impl->audio_.ballVel(),
-                                               M_impl->current_time_ );
-                break;
-            case PlayerMessage::PASS_INFO:
-                M_worldmodel.setHeardPassInfo( M_impl->audio_.receiverNumber(),
-                                               M_impl->audio_.receivePos(),
-                                               M_impl->current_time_ );
-                break;
-            case PlayerMessage::OPPONENT_GOALIE_INFO:
-                M_worldmodel.setHeardGoalieInfo( M_impl->audio_.goalieNumber(),
-                                                 M_impl->audio_.goaliePos(),
-                                                 M_impl->current_time_ );
-                break;
-            case PlayerMessage::OFFSIDE_LINE_INFO:
-                M_worldmodel.setHeardOffsideLineInfo( M_impl->audio_.offsideLineX(),
-                                                      M_impl->current_time_ );
-                break;
-            case PlayerMessage::DEFENSE_LINE_INFO:
-                M_worldmodel.setHeardDefenseLineInfo( M_impl->audio_.defenseLineX(),
-                                                      M_impl->current_time_ );
-                break;
-            case PlayerMessage::WAIT_REQUEST:
-                M_worldmodel.setHeardWaitRequest( M_impl->current_time_ );
-                break;
-            case PlayerMessage::ATTACK_REQUEST:
-                M_worldmodel.setHeardAttackRequest( M_impl->current_time_ );
-                break;
-            case PlayerMessage::INTERCEPT_INFO:
-                M_worldmodel.setHeardInterceptInfo( M_impl->audio_.interceptNumber(),
-                                                    M_impl->audio_.interceptCycle(),
-                                                    M_impl->current_time_ );
-                break;
-            case PlayerMessage::HEY_PASS:
-            case PlayerMessage::UNKNOWN:
-            default:
-                break;
-            }
-        }
-    }
+    M_impl->audio_.parsePlayerMessage( msg, M_impl->current_time_ );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1286,7 +1280,7 @@ PlayerAgent::analyzeHearOurCoach( const char * msg )
     dlog.addText( Logger::SENSOR,
                   "===receive say message from our coach" );
 
-    M_impl->audio_.parseCoach( msg, M_impl->current_time_ );
+    M_impl->audio_.parseCoachMessage( msg, M_impl->current_time_ );
 }
 
 
@@ -1312,7 +1306,7 @@ PlayerAgent::analyzeHearTrainer( const char * msg )
     dlog.addText( Logger::SENSOR,
                   "===receive trainer audio" );
 
-    M_impl->audio_.parseTrainer( msg, M_impl->current_time_ );
+    M_impl->audio_.parseTrainerMessage( msg, M_impl->current_time_ );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1572,6 +1566,14 @@ PlayerAgent::analyzeInit( const char * msg )
             com.toStr( ostrm );
         }
 
+        // set compression level
+        if ( 0 < config().compression()
+             && config().compression() <= 9 )
+        {
+            PlayerCompressionCommand com( config().compression() );
+            com.toStr( ostrm );
+        }
+
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s]",
                       ostrm.str().c_str() );
@@ -1653,7 +1655,11 @@ PlayerAgent::analyzeOK( const char * msg )
         int vermin, vermax;
         if ( std::sscanf( msg, "(ok clang (ver %d %d))", &vermin, &vermax ) == 2 )
         {
-            //std::cerr << "set clang version " << vermin << " " << vermax << std::endl;
+            //std::cerr << config().teamName() << ' '
+            //          << world().self().unum() << ": "
+            //          << M_impl->current_time_
+            //          << "set clang version " << vermin << " " << vermax
+            //          << std::endl;
             M_impl->clang_min_ = vermin;
             M_impl->clang_max_ = vermax;
             return;
@@ -1665,7 +1671,7 @@ PlayerAgent::analyzeOK( const char * msg )
               << M_impl->current_time_
               << " recv unsupported or illegal ok message [" << msg << "]" << std::endl;
     dlog.addText( Logger::SENSOR,
-                  "unsupported ok");
+                  "unsupported ok" );
 }
 
 /*-------------------------------------------------------------------*/
@@ -2041,6 +2047,13 @@ PlayerAgent::outputDebug()
     if ( config().debugConnect()
          || config().debugWrite() )
     {
+        // set say message
+        if ( ! effector().getSayMessage().empty() )
+        {
+            M_debug_client.addMessage( "Say[%s]",
+                                       effector().getSayMessage().c_str() );
+        }
+
         // set defense line
         M_debug_client.addLine( Vector2D( world().defenseLineX(),
                                           world().self().pos().y - 2.0 ),
@@ -2324,34 +2337,34 @@ PlayerAgent::doChangeView( const ViewWidth & width )
 /*!
 
 */
-bool
-PlayerAgent::doSay( const std::string & msg )
-{
-    if ( ! config().useCommunication() )
-    {
-        dlog.addText( Logger::ACTION,
-                      "agent->doSay. communication is not allowd" );
-        return false;
-    }
+// bool
+// PlayerAgent::doSay( const std::string & msg )
+// {
+//     if ( ! config().useCommunication() )
+//     {
+//         dlog.addText( Logger::ACTION,
+//                       "agent->doSay. communication is not allowed" );
+//         return false;
+//     }
 
-    // check message length
-    if ( static_cast< int >( msg.length() )
-         > ServerParam::i().playerSayMsgSize() )
-    {
-        std::cerr << world().teamName() << ' '
-                  << world().self().unum() << ": "
-                  << world().time()
-                  << "  say message too long [" << msg << "]="
-                  << msg.length() << std::endl;
-        dlog.addText( Logger::ACTION,
-                      "agent->doSay. message too long. length= %d",
-                      msg.length() );
-        return false;
-    }
+//     // check message length
+//     if ( static_cast< int >( msg.length() )
+//          > ServerParam::i().playerSayMsgSize() )
+//     {
+//         std::cerr << world().teamName() << ' '
+//                   << world().self().unum() << ": "
+//                   << world().time()
+//                   << "  say message too long [" << msg << "]="
+//                   << msg.length() << std::endl;
+//         dlog.addText( Logger::ACTION,
+//                       "agent->doSay. message too long. length= %d",
+//                       msg.length() );
+//         return false;
+//     }
 
-    M_effector.setSay( msg, config().version() );
-    return true;
-}
+//     M_effector.setSay( msg, config().version() );
+//     return true;
+// }
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -2506,6 +2519,33 @@ PlayerAgent::setViewAction( ViewAction * act )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+PlayerAgent::addSayMessage( const SayMessage * message )
+{
+    if ( ! config().useCommunication() )
+    {
+        dlog.addText( Logger::ACTION,
+                      "agent->addSayMessage. communication is not allowed" );
+        return;
+    }
+
+    M_effector.addSayMessage( message );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+PlayerAgent::removeSayMessage( const char header )
+{
+    return M_effector.removeSayMessage( header );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerAgent::setIntention( SoccerIntention * intention )
