@@ -42,6 +42,9 @@
 #include <rcsc/types.h>
 
 #include <boost/scoped_ptr.hpp>
+#include <boost/shared_ptr.hpp>
+
+#include <typeinfo>
 
 namespace rcsc {
 
@@ -54,6 +57,8 @@ class SeeState;
 class ArmAction;
 class NeckAction;
 class ViewAction;
+class SayMessage;
+class SayMessageParser;
 class SoccerIntention;
 
 struct PlayerAgentImpl;
@@ -251,7 +256,7 @@ public:
       \param msg message string
       \return true if successfully registered.
     */
-    bool doSay( const std::string & msg );
+    //bool doSay( const std::string & msg );
 
     /*!
       \brief register pointto command.
@@ -302,6 +307,18 @@ public:
     */
     void setViewAction( ViewAction * act );
 
+    /*!
+      \brief add say message to the action effector
+      \param message pointer to the say mesage builder. this must be a dynamically allocated object.
+     */
+    void addSayMessage( const SayMessage * message );
+
+    /*!
+      \brief remove the registered say message if exist
+      \param header message header character
+      \return true if removed
+     */
+    bool removeSayMessage( const char header );
 
     /*!
       \brief set intention object
@@ -324,8 +341,8 @@ protected:
       \param cmd_parser command line parser object
       \return only if "help" option is given, false is returned.
 
-      This method is called from BasicClient::init( argc, argv )
-      BasicClient::init(argc,argv) should be called in main().
+      This method is called from SoccerAgent::init( argc, argv )
+      SoccerAgent::init(argc,argv) should be called in main().
     */
     virtual
     bool initImpl( CmdLineParser & cmd_parser );
@@ -368,6 +385,18 @@ protected:
     virtual
     void handleExit();
 
+    /*!
+      \brief register new say message parser object
+      \param parser pointer to the say mesage parser. this must be a dynamically allocated object.
+     */
+    void addSayMessageParser( boost::shared_ptr< SayMessageParser > parser );
+
+    /*!
+      \brief remove registered parser object
+      \param header say message header character
+     */
+    void removeSayMessageParser( const char header );
+
 public:
     /*!
       \brief finalize all things when program quits
@@ -417,118 +446,117 @@ private:
 
     /*!
       \brief analyze see info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeSee( const char * msg );
 
     /*!
       \brief analyze sense_body info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeSenseBody( const char * msg );
 
     /*!
       \brief analyze hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHear( const char * msg );
 
     /*!
       \brief analyze referee message inf hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearReferee( const char * msg );
 
     /*!
       \brief analyze player message in hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearPlayer( const char * msg );
 
     /*!
       \brief analyze our coach message in hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearOurCoach( const char * msg );
 
     /*!
       \brief analyze opponent coach message in hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearOpponentCoach( const char * msg );
 
     /*!
       \brief analyze trainer message in hear info
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeHearTrainer( const char * msg );
 
     /*!
       \brief analyze fullstate message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeFullstate( const char * msg );
 
     /*!
       \brief analyze player_type message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzePlayerType( const char * msg );
 
     /*!
       \brief analyze player_param message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzePlayerParam( const char * msg );
 
     /*!
       \brief analyze server_param message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeServerParam( const char * msg );
 
-
     /*!
       \brief analyze init replay message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeInit( const char * msg );
 
     /*!
       \brief analyze change_player_type message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeChangePlayerType( const char * msg );
 
 
     /*!
       \brief analyze score message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeScore( const char * msg );
 
     /*!
       \brief analyze ok message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeOK( const char * msg );
 
     /*!
       \brief analyze error message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeError( const char * msg );
 
     /*!
       \brief analyze warningok message
-      \paramm msg raw server message
+      \param msg raw server message
     */
     void analyzeWarning( const char * msg );
 
     ////////////////////////////////////
 
     /*!
-      \brief main action decition. this method calls doAction()
+      \brief main action decision. this method calls doAction()
     */
     void action();
 
