@@ -204,13 +204,16 @@ PlayerChangeViewCommand::toStr( std::ostream & to ) const
 std::ostream &
 PlayerSayCommand::toStr( std::ostream & to ) const
 {
-    if ( M_version >= 8.0 )
+    if ( ! M_message.empty() )
     {
-        to << "(say \"" << M_message << "\")";
-    }
-    else
-    {
-        to << "(say " << M_message << ")";
+        if ( M_version >= 8.0 )
+        {
+            to << "(say \"" << M_message << "\")";
+        }
+        else
+        {
+            to << "(say " << M_message << ")";
+        }
     }
     return to;
 }
