@@ -33,6 +33,7 @@
 #define RCSC_PLAYER_COMMAND_H
 
 #include <rcsc/player/view_mode.h>
+#include <rcsc/geom/vector_2d.h>
 #include <rcsc/geom/angle_deg.h>
 
 #include <string>
@@ -321,6 +322,77 @@ public:
     */
     virtual
     std::string name() const = 0;
+
+
+    /*!
+      \brief get move command parameter
+      \return move position
+     */
+    virtual
+    Vector2D movePos() const
+      {
+          return Vector2D::INVALIDATED;
+      }
+
+    /*!
+      \brief get dash command parameter
+      \return dash power
+     */
+    virtual
+    double dashPower() const
+      {
+          return 0.0;
+      }
+
+    /*!
+      \brief get turn command parameter
+      \return turn moment
+     */
+    virtual
+    double turnMoment() const
+      {
+          return 0.0;
+      }
+
+    /*!
+      \brief get kick command parameter
+      \return kick power
+     */
+    virtual
+    double kickPower() const
+      {
+          return 0.0;
+      }
+
+    /*!
+      \brief get kick command parameter
+      \return kick direction
+     */
+    virtual
+    double kickDir() const
+      {
+          return 0.0;
+      }
+
+    /*!
+      \brief get catch command parameter
+      \return catch direction
+     */
+    virtual
+    double catchDir() const
+      {
+          return 0.0;
+      }
+
+    /*!
+      \brief get tackle command parameter
+      \return tackle power
+     */
+    virtual
+    double tacklePower() const
+      {
+          return 0.0;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -377,6 +449,16 @@ public:
       {
           return std::string( "move" );
       }
+
+    /*!
+      \brief get move command parameter
+      \return move position
+     */
+    Vector2D movePos() const
+      {
+          return Vector2D( M_x, M_y );
+      }
+
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -427,6 +509,15 @@ public:
       {
           return std::string( "dash" );
       }
+
+    /*!
+      \brief get dash command parameter
+      \return dash power
+     */
+    double dashPower() const
+      {
+          return M_power;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -477,6 +568,15 @@ public:
       {
           return std::string( "turn" );
       }
+
+    /*!
+      \brief get turn command parameter
+      \return turn moment
+     */
+    double turnMoment() const
+      {
+          return M_moment;
+      }
 };
 
 
@@ -531,6 +631,24 @@ public:
       {
           return std::string( "kick" );
       }
+
+    /*!
+      \brief get kick command parameter
+      \return kick power
+     */
+    double kickPower() const
+      {
+          return M_power;
+      }
+
+    /*!
+      \brief get kick command parameter
+      \return kick direction
+     */
+    double kickDir() const
+      {
+          return M_dir;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -581,6 +699,15 @@ public:
       {
           return std::string( "catch" );
       }
+
+    /*!
+      \brief get catch command parameter
+      \return catch direction
+     */
+    double catchDir() const
+      {
+          return M_dir;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -631,6 +758,16 @@ public:
       {
           return std::string( "tackle" );
       }
+
+    /*!
+      \brief get tackle command parameter
+      \return tackle power
+     */
+    virtual
+    double tacklePower() const
+      {
+          return M_power;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -835,6 +972,16 @@ private:
     std::string M_message; //!< message body
     double M_version; //!< client version
 public:
+
+    /*!
+      \brief construct with client version, no message
+      \param version player's client version
+     */
+    explicit
+    PlayerSayCommand( const double & version )
+        : M_version( version )
+      { }
+
     /*!
       \brief construct with cstring say message
       \param msg cstring message
@@ -881,6 +1028,34 @@ public:
           return std::string( "say" );
       }
 
+    /*!
+      \brief assign new message
+      \param msg new string
+     */
+    void assign( const std::string & msg )
+      {
+          M_message = msg;
+      }
+
+    /*!
+      \brief append new string
+      \param msg appended string
+     */
+    void append( const std::string & msg )
+      {
+          M_message += msg;
+      }
+
+    /*!
+      \brief get command parameter
+      \return say message
+     */
+    const
+    std::string & message() const
+      {
+          return M_message;
+      }
+
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -947,6 +1122,35 @@ public:
           return std::string( "pointto" );
       }
 
+
+    /*!
+      \brief get pointto command switch
+      \return true if pointo is on
+     */
+    bool pointtoOn() const
+      {
+          return M_on;
+      }
+
+    /*!
+      \brief get pointto command parameter
+      \return pointto distance
+     */
+    const
+    double & pointtoDist() const
+      {
+          return M_dist;
+      }
+
+    /*!
+      \brief get pointto command parameter
+      \return pointto dir
+     */
+    const
+    double & pointtoDir() const
+      {
+          return M_dist;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -1103,6 +1307,24 @@ public:
       {
           return std::string( "clang" );
       }
+
+    /*!
+      \brief get clang command parameter
+      \return clang min version
+     */
+    int minVer() const
+      {
+          return M_min;
+      }
+
+    /*!
+      \brief get clang command parameter
+      \return clang max version
+     */
+    int maxVer() const
+      {
+          return M_max;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -1212,6 +1434,35 @@ public:
       {
           return std::string( "ear" );
       }
+
+
+    /*!
+      \brief get ear command parameter
+      \return on/off type
+    */
+    OnOffType onOff() const
+      {
+          return M_onoff;
+      }
+
+    /*!
+      \brief get ear command parameter
+      \return side type
+     */
+    SideType side() const
+      {
+          return M_side;
+      }
+
+    /*!
+      \brief get ear command parameter
+      \return mode type
+     */
+    ModeType mode() const
+      {
+          return M_mode;
+      }
+
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -1364,6 +1615,15 @@ public:
       {
           return std::string( "compression" );
       }
+
+    /*!
+      \brief get compression command parameter
+      \return compression level
+     */
+    int level() const
+      {
+          return M_level;
+      }
 };
 
 
