@@ -69,7 +69,7 @@ void
 PlayerConfig::setDefaultParam()
 {
     // basic setting
-    M_team_name = "HELIOS";
+    M_team_name = "HELIOS2007";
     M_version = 9.4;
     M_reconnect_number = 0;
     M_goalie = false;
@@ -85,6 +85,8 @@ PlayerConfig::setDefaultParam()
     M_rcssserver_host = "localhost";
     M_rcssserver_port = 6000;
 
+    M_compression = -1;
+
     M_clang_min = 7;
     M_clang_max = 8;
 
@@ -164,6 +166,8 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "host", "h", &M_rcssserver_host )
         ( "port", "", &M_rcssserver_port )
 
+        ( "compression", "", &M_compression )
+
         ( "clang_min", "", &M_clang_min )
         ( "clang_max", "", &M_clang_max )
 
