@@ -66,6 +66,8 @@ private:
     std::string M_rcssserver_host; //!< host name that rcssserver is running
     int         M_rcssserver_port; //!< rcssserver connection port number
 
+    int M_compression; //!< zlib compression level for the compression command
+
     int M_clang_min; //!< supported clang min version
     int M_clang_max; //!< supported clang max version
 
@@ -210,6 +212,11 @@ public:
           return M_rcssserver_port;
       }
 
+    int compression() const
+      {
+          return M_compression;
+      }
+
     int clangMin() const
       {
           return M_clang_min;
