@@ -37,7 +37,8 @@
 #include "fullstate_sensor.h"
 #include "logger.h"
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_type.h>
 
 namespace rcsc {
 
@@ -50,15 +51,20 @@ int PlayerObject::S_face_count_thr = 2;
 
 */
 PlayerObject::PlayerObject()
-    : M_unum( -1 )
+    : M_unum( Unum_Unknown )
     , M_goalie( false )
+    , M_type( Hetero_Unknown )
+    , M_player_type_ptr( static_cast< const PlayerType * >( 0 ) )
     , M_dist_from_self( 1000.0 )
     , M_angle_from_self( 0.0 )
     , M_dist_from_ball( 1000.0 )
+    , M_ghost_count( 0 )
     , M_pos( Vector2D::INVALIDATED )
-    , M_rpos(  Vector2D::INVALIDATED )
+    , M_rpos( Vector2D::INVALIDATED )
     , M_pos_count( 1000 )
     , M_rpos_count( 1000 )
+    , M_prev_pos( Vector2D::INVALIDATED )
+    , M_prev_pos_count( 0 )
     , M_vel( 0.0, 0.0 )
     , M_vel_count( 1000 )
     , M_body( 0.0 )
@@ -79,12 +85,17 @@ PlayerObject::PlayerObject()
 PlayerObject::PlayerObject( const Localization::PlayerT & p )
     : M_unum( p.unum_ )
     , M_goalie( p.goalie_ )
+    , M_type( Hetero_Unknown )
+    , M_player_type_ptr( static_cast< const PlayerType * >( 0 ) )
     , M_dist_from_self( p.rpos_.r() )
     , M_angle_from_self( 0.0 )
     , M_dist_from_ball( 1000.0 )
+    , M_ghost_count( 0 )
     , M_pos( p.pos_ )
     , M_rpos( p.rpos_ )
     , M_pos_count( 0 )
+    , M_prev_pos( p.pos_ )
+    , M_prev_pos_count( 0 )
     , M_vel( 0.0, 0.0 )
     , M_vel_count( 1000 )
     , M_body( 0.0 )
@@ -100,6 +111,7 @@ PlayerObject::PlayerObject( const Localization::PlayerT & p )
         M_vel = p.vel_;
         M_vel_count = 0;
     }
+
     if ( p.hasAngle() )
     {
         M_body = p.body_;
@@ -107,11 +119,13 @@ PlayerObject::PlayerObject( const Localization::PlayerT & p )
         M_face = p.face_;
         M_face_count = 0;
     }
+
     if ( p.isPointing() )
     {
         M_pointto_angle = p.arm_;
         M_pointto_count = 0;
     }
+
     if ( p.isTackling() )
     {
         M_tackle_count = 0;
@@ -147,9 +161,14 @@ PlayerObject::isTackling() const
 
 */
 bool
-PlayerObject::isKickable() const
+PlayerObject::isKickable( const double & buf ) const
 {
-    return distFromBall() < ServerParam::i().defaultKickableArea();
+    if ( ! M_player_type_ptr )
+    {
+        return distFromBall() < ServerParam::i().defaultKickableArea();
+    }
+
+    return distFromBall() < M_player_type_ptr->kickableArea() - buf;
 }
 
 /*-------------------------------------------------------------------*/
@@ -172,7 +191,9 @@ PlayerObject::update()
     */
     ++M_pos_count;
     ++M_rpos_count;
+    ++M_prev_pos_count;
     ++M_vel_count;
+    ++M_body_count;
     ++M_face_count;
     ++M_pointto_count;
     ++M_tackle_count;
@@ -185,13 +206,15 @@ PlayerObject::update()
 void
 PlayerObject::updateBySee( const Localization::PlayerT & p )
 {
+    M_ghost_count = 0;
+
     // unum is updated only when unum is seen.
-    if ( p.unum_ != -1 )
+    if ( p.unum_ != Unum_Unknown )
     {
         M_unum = p.unum_;
         if ( ! p.goalie_ )
         {
-            // when unum seen, goalie info also seen
+            // when unum is seen, goalie info is also seen
             M_goalie = false;
         }
     }
@@ -206,22 +229,33 @@ PlayerObject::updateBySee( const Localization::PlayerT & p )
         M_vel = p.vel_;
         M_vel_count = 0;
         dlog.addText( Logger::WORLD,
-                      "player %d updateBySee. pos=(%f, %f) vel=(%f, %f)",
+                      "%s: updateBySee. unum=%d. pos=(%.2f, %.2f) vel=(%.2f, %.2f)"
+                      ,__FILE__,
                       p.unum_, M_pos.x, M_pos.y, M_vel.x, M_vel.y );
     }
-    else if ( M_pos_count <= 2 )
+    else if ( 0 < M_pos_count
+              && M_pos_count <= 2
+              && p.rpos_.r() < 40.0 )
     {
-        M_vel = ( p.pos_ - M_pos ) / ( static_cast< double >( M_pos_count ) + 0.001 );
+        const double speed_max = ( M_player_type_ptr
+                                   ? M_player_type_ptr->playerSpeedMax()
+                                   : ServerParam::i().defaultPlayerSpeedMax() );
+        const double decay = ( M_player_type_ptr
+                               ? M_player_type_ptr->playerDecay()
+                               : ServerParam::i().defaultPlayerSpeedMax() );
+
+        M_vel = ( p.pos_ - M_pos ) / static_cast< double >( M_pos_count );
         double tmp = M_vel.r();
-        if ( tmp > ServerParam::i().defaultPlayerSpeedMax() )
+        if ( tmp > speed_max )
         {
-            M_vel *= ServerParam::i().defaultPlayerSpeedMax() / tmp;
+            M_vel *= speed_max / tmp;
         }
-        M_vel *= ServerParam::i().defaultPlayerDecay();
+        M_vel *= decay;
         M_vel_count = M_pos_count;
         dlog.addText( Logger::WORLD,
-                      "player %d updateBySee. update vel by pos diff."
-                      "prev_pos=(%.2f, %.2f) old_pos=(%.2f, %.2f) -> vel=(%.3f, %.3f)",
+                      "%s: updateBySee. unum=%d. update vel by pos diff."
+                      "prev_pos=(%.2f, %.2f) old_pos=(%.2f, %.2f) -> vel=(%.3f, %.3f)"
+                      ,__FILE__,
                       p.unum_,
                       M_pos.x, M_pos.y, p.pos_.x, p.pos_.y, M_vel.x, M_vel.y );
     }
@@ -231,8 +265,10 @@ PlayerObject::updateBySee( const Localization::PlayerT & p )
         M_vel_count = 1000;
     }
 
+    M_prev_pos = M_pos;
     M_pos = p.pos_;
     M_rpos = p.rpos_;
+
     M_pos_count = 0;
     M_rpos_count = 0;
 
@@ -274,13 +310,18 @@ PlayerObject::updateByFullstate( const FullstateSensor::PlayerT & p,
                                  const Vector2D & self_pos,
                                  const Vector2D & ball_pos )
 {
+    M_ghost_count = 0;
+
     M_unum = p.unum_;
     M_goalie = p.goalie_;
 
     M_pos = p.pos_;
     M_rpos = p.pos_ - self_pos;
+    M_prev_pos = p.pos_;
+
     M_pos_count = 0;
     M_rpos_count = 0;
+    M_prev_pos_count = 0;
 
     M_vel = p.vel_;
     M_vel_count = 0;
@@ -307,14 +348,26 @@ PlayerObject::updateByHear( const int heard_unum,
                             const bool goalie,
                             const Vector2D & heard_pos )
 {
-    if ( posCount() > 1 )
+    M_ghost_count = 0;
+
+    if ( heard_unum != Unum_Unknown )
     {
         M_unum = heard_unum;
+    }
+
+    if ( goalie )
+    {
+        M_goalie = true;
+    }
+
+    if ( posCount() > 1
+         || distFromSelf() > 20.0 )
+    {
         M_pos = heard_pos;
-        M_pos_count = 1;
-        if ( goalie )
+
+        if ( posCount() > 1 )
         {
-            M_goalie = true;
+            M_pos_count = 1;
         }
     }
 }
@@ -322,6 +375,36 @@ PlayerObject::updateByHear( const int heard_unum,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+PlayerObject::updateByHear( const int heard_unum,
+                            const bool goalie,
+                            const Vector2D & heard_pos,
+                            const AngleDeg & heard_body )
+{
+    updateByHear( heard_unum, goalie, heard_pos );
+
+    if ( bodyCount() > 1 )
+    {
+        M_body = heard_body;
+        M_body_count = 1;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+PlayerObject::setPlayerType( const HeteroID id )
+{
+    M_type = id;
+    M_player_type_ptr = PlayerTypeSet::i().get( id );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerObject::updateSelfBallRelated( const Vector2D & self,
@@ -350,6 +433,7 @@ PlayerObject::forget()
         = M_pointto_count
         = M_tackle_count
         = 1000;
+    M_prev_pos_count = 0;
 }
 
 }
