@@ -210,7 +210,7 @@ SeeState::setNewCycle( const GameTime & new_time )
     }
 
     // std::rotate( [0], [1], [SIZE] );
-    for ( int i = HISTORY_SIZE - 1; i > 0; i-- )
+    for ( int i = HISTORY_SIZE - 1; i > 0; --i )
     {
         M_see_count_history[i] = M_see_count_history[i-1];
     }
