@@ -46,7 +46,7 @@
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
-#define DEBUG1
+//#define DEBUG1
 //#define DEBUG
 
 namespace rcsc {
@@ -498,14 +498,7 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
         = next_ball_rel.x
         + std::sqrt( std::pow( best_ctrl_dist - 0.1, 2.0 )
                      - std::pow( next_ball_rel.y, 2.0 ) );
-#if 0
-    // nearest side has priority
-    if ( std::fabs( required_accel_x[1] )
-         < std::fabs( required_accel_x[0] ) )
-    {
-        std::swap( required_accel_x[0], required_accel_x[1] );
-    }
-#else
+
     if ( M_world.self().body().abs() < 45.0 )
     {
         // forward dash has priority if player's body face to opponent side
@@ -514,7 +507,12 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
             std::swap( required_accel_x[0], required_accel_x[1] );
         }
     }
-#endif
+    else if ( std::fabs( required_accel_x[1] )
+              < std::fabs( required_accel_x[0] ) )
+    {
+        // nearest side has priority
+        std::swap( required_accel_x[0], required_accel_x[1] );
+    }
 
 #ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
@@ -1163,8 +1161,8 @@ SelfIntercept::canReachAfterDash( const int n_turn,
         double player_travel = tmp_pos.r();
         double player_noise = player_travel * ServerParam::i().playerRand() * 0.5;
         double last_ball_dist = ball_rel.dist( tmp_pos );
-        double buf = 0.0;
-        if ( n_turn > 0 ) buf += 0.2;
+        double buf = 0.2;
+        if ( n_turn > 0 ) buf += 0.3;
         buf += player_noise;
         buf += ball_noise;
         if ( last_ball_dist < control_area - buf )
