@@ -42,7 +42,7 @@
 #include "fullstate_sensor.h"
 #include "logger.h"
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/soccer_math.h>
 
 #include <algorithm>
@@ -70,7 +70,7 @@ SelfObject::SelfObject()
     , M_pos_prev( Vector2D::INVALIDATED )
     , M_vel( 0.0, 0.0 )
     , M_vel_error( 0.0, 0.0 )
-    , M_vel_count( 0 )
+    , M_vel_count( 1000 )
     , M_body( 0.0 )
     , M_neck( 0.0 )
     , M_face( 0.0 )
@@ -193,8 +193,8 @@ SelfObject::update( const ActionEffector & act,
 
     M_pos_prev = M_pos;
 
-    Vector2D acc( 0.0, 0.0 );
-    double dash_pow = 0.0;
+    Vector2D accel( 0.0, 0.0 );
+    double dash_power = 0.0;
     double turn_moment = 0.0, turn_err = 0.0;
     double neck_moment = 0.0;
 
@@ -202,7 +202,7 @@ SelfObject::update( const ActionEffector & act,
     // base command
     switch ( act.lastBodyCommandType() ) {
     case PlayerCommand::DASH:
-        act.getDashInfo( &acc, &dash_pow );
+        act.getDashInfo( &accel, &dash_power );
         break;
     case PlayerCommand::TURN:
         act.getTurnInfo( &turn_moment, &turn_err );
@@ -250,7 +250,7 @@ SelfObject::update( const ActionEffector & act,
     }
 
     // stamina
-    M_stamina.update( playerType(), dash_pow );
+    M_stamina.update( playerType(), dash_power );
 
 
     /////////////////////////////////////////////////////////////////
@@ -266,7 +266,7 @@ SelfObject::update( const ActionEffector & act,
     // vel
     if ( velValid() )
     {
-        M_vel += acc;
+        M_vel += accel;
     }
 
     /////////////////////////////////////////////////////////////////
@@ -293,7 +293,7 @@ SelfObject::update( const ActionEffector & act,
     // acc *= Zero;
 
     /////////////////////////////////////////////////////////////////
-    // error
+    // update error
     if ( velValid() )
     {
         const double velrnd = ServerParam::i().playerRand() * M_vel.r();
@@ -303,8 +303,9 @@ SelfObject::update( const ActionEffector & act,
         M_vel_error.add( velrnd, velrnd ); // add new vel rand
 
         dlog.addText( Logger::WORLD,
-                      "self update. pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
-                      "vel=(%.2f, %.2f) vel_err=(%.3f, %.3f)",
+                      "%s: update. pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
+                      "vel=(%.2f, %.2f) vel_err=(%.3f, %.3f)"
+                      ,__FILE__,
                       M_pos.x, M_pos.y, M_pos_error.x, M_pos_error.y,
                       M_vel.x, M_vel.y, M_vel_error.x, M_vel_error.y );
     }
@@ -318,10 +319,12 @@ SelfObject::update( const ActionEffector & act,
     M_vel_error *= this->playerType().playerDecay();
 
     /////////////////////////////////////////////////////////////////
-    // conf decay
-    M_pos_count++;
-    M_vel_count++;
-    M_face_count++;
+    // update accuracy count
+    ++M_pos_count;
+    ++M_vel_count;
+    ++M_face_count;
+
+    // update action effect count
     M_tackle_expires = std::max( 0, M_tackle_expires - 1 );
     M_arm_movable = std::max( 0, M_arm_movable - 1 );
 
@@ -497,7 +500,8 @@ SelfObject::updateAfterSense( const BodySensor & sense,
         M_vel_error.assign( ( maxcos - mincos ) * ( sense.speedMag() + 0.005 ),
                             ( maxsin - minsin ) * ( sense.speedMag() + 0.005 ) );
         dlog.addText( Logger::WORLD,
-                      "self.updateAfterSense. vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f",
+                      "%s: updateAfterSense. vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f"
+                      ,__FILE__,
                       M_vel.x, M_vel.y,
                       M_vel_error.x, M_vel_error.y,
                       faceError() );
@@ -505,10 +509,14 @@ SelfObject::updateAfterSense( const BodySensor & sense,
         ////////////////////////////////////////////////////
         // collision check
         if ( velValid()
-             && sense.speedMag() < playerType().realSpeedMax() * 0.11 )
+             && sense.speedMag() < ( playerType().realSpeedMax()
+                                     * playerType().playerDecay()
+                                     * 0.11 )
+             )
         {
             dlog.addText( Logger::WORLD,
-                          "self.updateAfterSense. check collisiion. estimate_vel=(%.2f %.2f)",
+                          "%s: updateAfterSense. check collisiion. estimate_vel=(%.2f %.2f)"
+                          ,__FILE__,
                           estimate_vel.x, estimate_vel.y );
             // decay is very small
             //   default decay = 0.4
@@ -524,15 +532,15 @@ SelfObject::updateAfterSense( const BodySensor & sense,
             {
                 M_collision_estimated = true;
                 dlog.addText( Logger::WORLD,
-                              "self.updateAfterSense. *COLLISION* estimated" );
+                              "%s: self.updateAfterSense. *COLLISION* estimated"
+                              ,__FILE__ );
             }
         }
 
         ////////////////////////////////////////////////////
-        // vel conf is face angle conf
+        // vel accuracy is face angle conf
         M_vel_count = faceCount();
 
-
         //////////////////////////////////////////////////////
         // pointto
         if ( posValid() )
@@ -543,7 +551,6 @@ SelfObject::updateAfterSense( const BodySensor & sense,
                                           face() + sense.pointtoDir() );
         }
 
-
         ////////////////////////////////////////////////////
         // last move
         if ( ! M_collision_estimated )
@@ -569,13 +576,19 @@ SelfObject::updateAfterSense( const BodySensor & sense,
     if ( M_tackle_expires > 0 )
     {
         dlog.addText( Logger::WORLD,
-                      "self.updateAfterSense internal_tackle_exp= %d,  sense_body= %d",
+                      "%s: updateAfterSense internal_tackle_exp=%d, sense_body=%d"
+                      ,__FILE__,
                       M_tackle_expires, sense.tackleExpires() );
     }
+
     if ( M_tackle_expires != sense.tackleExpires() )
     {
+        std::cerr << current
+                  << "self : updateAfterSense. internal tackle expire has invalie value."
+                  << std::endl;
         dlog.addText( Logger::WORLD,
-                      "self.updateAfterSense internal_tackle_exp is not match to sense_body" );
+                      "%s: updateAfterSense internal_tackle_exp is not match to sense_body"
+                      ,__FILE__ );
     }
 
     M_tackle_expires = sense.tackleExpires();
@@ -646,7 +659,8 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.x - pos.x ) * ( pos_err.x / ( M_pos_error.x + pos_err.x ) );
             new_error.x = ( M_pos_error.x + pos_err.x ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          "self.updatePos. X modified [%f] < [%f]",
+                          "%s: updatePos. X modified [%f] < [%f]"
+                          ,__FILE__,
                           M_pos_error.x, pos_err.x );
         }
         if  ( M_pos_error.y < pos_err.y )
@@ -656,15 +670,17 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.y - pos.y ) * ( pos_err.y / ( M_pos_error.y + pos_err.y ) );
             new_error.y = ( M_pos_error.y + pos_err.y ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          "self.updatePos. Y modified [%f] < [%f]",
+                          "%s: updatePos. Y modified [%f] < [%f]"
+                          ,__FILE__,
                           M_pos_error.y, pos_err.y );
         }
         M_pos = new_pos;
         M_pos_error = new_error;
 
         dlog.addText( Logger::WORLD,
-                      "self.updatePos. seen_pos=(%f, %f) average_pos=(%f, %f) "
-                      " error=(%f, %f)",
+                      "%s: updatePos. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
+                      " error=(%.4f, %.4f)"
+                      ,__FILE__,
                       pos.x, pos.y, new_pos.x, new_pos.y,
                       M_pos_error.x, M_pos_error.y );
     }
@@ -741,9 +757,6 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
         // vel error
         double mincos, maxcos, minsin, maxsin;
         // vel angle error is face error + sensed speed dir error
-        dlog.addText( Logger::WORLD,
-                      "self.updateVelDirAfterSee. face_error= %f,  sensed_dir_error = %f",
-                      faceError(), sensed_speed_dir_error );
         vel_ang.cosMinMax( faceError() + sensed_speed_dir_error,
                            &mincos,
                            &maxcos );
@@ -753,14 +766,18 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
         // sensed speed is rounded by 0.01
         M_vel_error.assign( ( maxcos - mincos ) * ( sense.speedMag() + 0.005 ),
                             ( maxsin - minsin ) * ( sense.speedMag() + 0.005 ) );
-        dlog.addText( Logger::WORLD,
-                      "self.updateVelDirAfterSee. vel=(%f %f) )vel_error=(%f, %f)",
-                      M_vel.x, M_vel.y,
-                      M_vel_error.x, M_vel_error.y);
 
         // set last move
         M_last_move = M_vel / this->playerType().playerDecay();
 
+        dlog.addText( Logger::WORLD,
+                      "%s: updateVelDirAfterSee. face_error=%.2f, sensed_dir_error=%.2f."
+                      " vel=(%f %f) vel_err=(%f %f) last_move=(%.4f %.4f)"
+                      ,__FILE__,
+                      faceError(), sensed_speed_dir_error,
+                      M_vel.x, M_vel.y,
+                      M_vel_error.x, M_vel_error.y,
+                      M_last_move.x, M_last_move.y );
     }
 }
 
@@ -782,9 +799,7 @@ SelfObject::updateBallInfo( const BallObject & ball )
 
     //------------------------------------------------------
     // check kickable
-    dlog.addText( Logger::WORLD,
-                  "self.updateBallInfo. ball_dist= %f.  my_kickable= %f",
-                  ball.distFromSelf(), kickableArea() );
+
     if ( ball.distFromSelf() <= this->kickableArea() - 0.06 )
     {
         M_kickable = true;
@@ -828,9 +843,11 @@ SelfObject::updateBallInfo( const BallObject & ball )
         // else FAILURE;
         M_tackle_probability = 1.0 - fail_prob;
     }
+
     dlog.addText( Logger::WORLD,
-                  "self.updateBallInfo. tackle_prob= %f",
-                  M_tackle_probability );
+                  "%s: updateBallInfo. ball_dist=%.3f.  my_kickable=%.3f.  tackle_prob=%.3f"
+                  ,__FILE__,
+                  ball.distFromSelf(), kickableArea(), M_tackle_probability );
 }
 
 /*-------------------------------------------------------------------*/
@@ -841,8 +858,9 @@ double
 SelfObject::getSafetyDashPower( const double & dash_power ) const
 {
     // required stamina value
-    double required_stamina = dash_power;
-    if ( dash_power < 0.0 ) required_stamina *= -2.0;
+    double required_stamina = ( dash_power > 0.0
+                                ? dash_power
+                                : dash_power * -2.0 );
 
     if ( required_stamina < 0.0 )
     {
@@ -859,9 +877,7 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
     const double safety_stamina
         = stamina() - ServerParam::i().effortDecThrValue() - 3.0;
 
-    double available_stamina = ( safety_stamina < 0.0
-                                 ? 0.0
-                                 : safety_stamina );
+    double available_stamina = std::max( 0.0, safety_stamina );
 
     double result_power = std::min( required_stamina, available_stamina );
     // backward dash case
@@ -870,31 +886,31 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
         result_power *= -0.5;
     }
 
+    if ( std::fabs( result_power ) > std::fabs( dash_power ) )
+    {
+        return dash_power;
+    }
+
     dlog.addText( Logger::ACTION,
                   "%s:%d: getSafetyDashPower. cur_stamina=%.1f, required=%.1f, result=%.2f"
                   ,__FILE__, __LINE__,
                   stamina(), required_stamina, result_power );
-#ifdef DEBUG
-    if ( dash_power * result_power < 0.0 )
-    {
-        std::cerr << "get_recovery_save_power. reversed? input= " << dash_power
-                  << "  result = " << result_power << std::endl;
-        dlog.addText( Logger::ACTION,
-                      "%s:%d: get_recovery_save_power. reversed? input= %f result= %f"
-                      ,__FILE__, __LINE__,
-                      dash_power, result_power );
-    }
 
-    if ( result_power < ServerParam::i().minPower()
-         || ServerParam::i().maxPower() < result_power )
-    {
-        std::cerr << "get_recovery_save_power. over power range" << std::endl;
-        dlog.addText( Logger::ACTION,
-                      "%s:%dL get_recovery_save_power. over power range"
-                      ,__FILE__, __LINE__ );
-    }
-#endif
+
     return result_power;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+SelfObject::canTurn( const double & moment ) const
+{
+    return std::fabs( moment )
+        <= playerType().effectiveTurn( ServerParam::i().maxMoment(),
+                                       vel().r() );
+}
+
 }
