@@ -667,6 +667,33 @@ public:
     */
     double getSafetyDashPower( const double & dash_power ) const;
 
+    /*!
+      \brief check if agent can turn the specified moment
+      \param moment required turn moment value
+      \return true if agent can turn by 1 step
+     */
+    bool canTurn( const double & moment ) const;
+
+
+    /*!
+      \brief estimate reach point
+      \param n_step this method estimates ball point after this steps
+      \return estimated point vector
+    */
+    Vector2D inertiaPoint( const int n_step ) const
+      {
+          return playerType().inertiaPoint( pos(), vel(), n_step );
+      }
+
+    /*!
+      \brief estimate final reach point
+      \return estimated point vector
+    */
+    Vector2D inertiaFinalPoint() const
+      {
+          return playerType().inertiaFinalPoint( pos(), vel() );
+      }
+
 };
 
 }
