@@ -37,9 +37,11 @@
 
 #include "logger.h"
 
+#include <cstdlib>
+#include <cmath> // HUGE_VAL
 #include <iterator>
 #include <algorithm>
-#include <cmath> // HUGE_VAL
+#include <limits> // std::numeric_limits
 
 namespace rcsc {
 
@@ -147,7 +149,7 @@ operator<<( std::ostream & os,
 
 /*-------------------------------------------------------------------*/
 
-const double VisualSensor::DIST_ERR = HUGE_VAL;
+const double VisualSensor::DIST_ERR = std::numeric_limits< double >::max();
 const double VisualSensor::DIR_ERR = -360;
 
 /*-------------------------------------------------------------------*/
@@ -819,6 +821,7 @@ VisualSensor::parsePlayer( const char * tok,
     {
         while ( *tok != '\"' ) ++tok; // " skip to team name
         ++tok; // skip '"'
+
         if ( *( tok + team_name_len ) == '\"' // "
              && ! std::strncmp( team_name, tok, team_name_len ) )
         {
@@ -827,14 +830,12 @@ VisualSensor::parsePlayer( const char * tok,
         else
         {
             result_type = Player_Unknown_Opponent;
-            /*
-              if ( M_opponent_team_name.empty() )
-              {
-              while (*tok != '\"') M_opponent_team_name += *tok++; // "
-              // std::cerr << "copy opponent team name : "
-              // << M_opponent_team_name << std::endl;
-              }
-            */
+            if ( M_opponent_team_name.empty() )
+            {
+                while (*tok != '\"') M_opponent_team_name += *tok++; // "
+                // std::cerr << "copy opponent team name : "
+                // << M_opponent_team_name << std::endl;
+            }
         }
     }
     else
@@ -854,7 +855,7 @@ VisualSensor::parsePlayer( const char * tok,
     }
     else
     {
-        info->unum_ = -1;
+        info->unum_ = Unum_Unknown;
     }
 
     // check goalie flag
