@@ -41,20 +41,23 @@
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
 
+#include <boost/shared_ptr.hpp>
+
 #include <list>
 #include <vector>
 #include <string>
 
 namespace rcsc {
 
-class PlayerType;
+class AudioMemory;
 class ActionEffector;
 class BodySensor;
-class VisualSensor;
 class FullstateSensor;
-class Localization;
 class InterceptTable;
+class Localization;
+class PlayerType;
 class PenaltyKickState;
+class VisualSensor;
 struct ViewArea;
 
 /*!
@@ -71,15 +74,18 @@ public:
     static const double DIR_STEP;
 
 private:
-    // internal implementation
+
     Localization * M_localize; //!< localization module
     InterceptTable * M_intercept_table; //!< interception info table
+    boost::shared_ptr< AudioMemory > M_audio_memory; //!< heard info memory
     PenaltyKickState * M_penalty_kick_state; //!< penalty kick mode status
 
     //////////////////////////////////////////////////
     std::string M_teamname; //!< our teamname
     SideID M_our_side; //!< our side ID
 
+    std::string M_opponent_teamname; //!< opponent teamname
+
     //////////////////////////////////////////////////
     // timer & mode
 
@@ -117,6 +123,8 @@ private:
     //! sorted by distance from ball. include unknown players
     PlayerPtrCont M_opponents_from_ball;
 
+    //! uniform number of opponent goalie
+    int M_opponent_goalie_unum;
 
     //////////////////////////////////////////////////
     // strategic value
@@ -142,31 +150,6 @@ private:
     //! array of direction confidence count
     int M_dir_count[DIR_CONF_DIVS];
 
-    //////////////////////////////////////////////////
-    // pass info
-    int M_heard_receiver_number; //!< heard pass receiver number
-    Vector2D M_heard_receive_pos; //!< heard pass receive pos
-    GameTime M_heard_pass_time; //!< heard pass start time
-
-    //////////////////////////////////////////////////
-    // audio goalie info
-    Vector2D M_heard_goalie_pos;
-    GameTime M_heard_goalie_time;
-
-    double M_heard_offside_line_x;
-    GameTime M_heard_offside_line_time;
-
-    double M_heard_defense_line_x;
-    GameTime M_heard_defense_line_time;
-
-    GameTime M_heard_wait_request_time;
-
-    GameTime M_heard_attack_request_time;
-
-    int M_heard_intercept_unum;
-    int M_heard_intercept_cycle;
-    GameTime M_heard_intercept_time;
-
     //////////////////////////////////////////////////
 
     //! not used
@@ -199,6 +182,16 @@ public:
     const
     PenaltyKickState * penaltyKickState() const;
 
+    /*!
+      \brief get audio memory
+      \return co
+     */
+    const
+    AudioMemory & audioMemory() const
+      {
+          return *M_audio_memory;
+      }
+
     /*!
       \brief init team info
       \param teamname our team name string
@@ -214,6 +207,13 @@ public:
                        const int my_unum,
                        const bool my_goalie );
 
+    /*!
+      \brief set new audio memory
+      \param memory pointer to the memory instance. This must be
+      a dynamically allocated object.
+     */
+    void setAudioMemory( boost::shared_ptr< AudioMemory > memory );
+
     /*!
       \brief set teammate player type
       \param unum uniform number of changed teammate
@@ -300,50 +300,6 @@ public:
           M_self.setViewMode( w, q );
       }
 
-    /*!
-      \brief set heard ball status
-      \param ball_pos heard ball posision
-      \param ball_vel heard ball velocity
-      \param current current game time
-    */
-    void setHeardBallInfo( const Vector2D & ball_pos,
-                           const Vector2D & ball_vel,
-                           const GameTime & current );
-
-    /*!
-      \brief set heard pass info
-      \param receiver receiver number
-      \param recv_pos pass receive point
-      \param current current game time
-    */
-    void setHeardPassInfo( const int receiver,
-                           const Vector2D & recv_pos,
-                           const GameTime & current );
-
-    /*!
-      \brief set heard opponent goalie info
-      \param goalie_number heard opponent goalie uniform number
-      \param goalie_pos heard opponent goalie pos
-      \param current current game time
-    */
-    void setHeardGoalieInfo( const int goalie_number,
-                             const Vector2D & goalie_pos,
-                             const GameTime & current );
-
-    void setHeardOffsideLineInfo( const double & offside_line_x,
-                                  const GameTime & current );
-
-    void setHeardDefenseLineInfo( const double & defense_line_x,
-                                  const GameTime & current );
-
-    void setHeardWaitRequest( const GameTime & current );
-
-    void setHeardAttackRequest( const GameTime & current );
-
-    void setHeardInterceptInfo( const int unum,
-                                const int cycle,
-                                const GameTime & current );
-
     /*!
       \brief internal update for action decision
       \param act action effector
@@ -438,6 +394,21 @@ private:
     */
     void updateDirCount( const ViewArea & varea );
 
+    /*!
+      \brief update opponent goalie by heard info
+    */
+    void updateGoalieByHear();
+
+    /*!
+      \brief update opponent goalie by heard info
+    */
+    void updatePlayerByHear();
+
+    /*!
+      \brief update player type id of the recognized players
+     */
+    void updatePlayerType();
+
     /*!
       \brief update object relation
 
@@ -481,6 +452,25 @@ public:
           return M_our_side;
       }
 
+    /*!
+      \brief get opponent teamname
+      \return const reference to the team name string
+    */
+    const
+    std::string & opponentTeamName() const
+      {
+          return M_opponent_teamname;
+      }
+
+    /*!
+      \brief get opponent team side Id
+      \return side Id
+    */
+    SideID theirSide() const
+      {
+          return M_our_side == LEFT ? RIGHT : LEFT;
+      }
+
     /*!
       \brief check if our team is left or not
       \return true if our team is left side
@@ -519,6 +509,16 @@ public:
           return M_see_time;
       }
 
+    /*!
+       \brief get last time updated by fullstate
+       \return const reference to the game time object
+     */
+    const
+    GameTime & fullstateTime() const
+      {
+          return M_fullstate_time;
+      }
+
     /*!
       \brief get last time updated by fullstate
      */
@@ -645,17 +645,26 @@ public:
           return M_opponents_from_ball;
       }
 
+    /*!
+      \brief get the uniform number of opponent goalie
+      \return uniform number value or Unum_Unknown
+     */
+    int opponentGoalieUnum() const
+      {
+          return M_opponent_goalie_unum;
+      }
+
     //////////////////////////////////////////////////////////
 
     /*!
       \brief get number specified teammate player object pointer
-      \param unum wanted player"s uniform number
+      \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
     const
     PlayerObject * getTeammate( const int unum ) const
       {
-          PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
+          const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
           for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
                 it != end;
                 ++it )
@@ -670,13 +679,13 @@ public:
 
     /*!
       \brief get number specified opponent player object pointer
-      \param unum wanted player"s uniform number
+      \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
     const
     PlayerObject * getOpponent( const int unum ) const
       {
-          PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
+          const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
           for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
                 it != end;
                 ++it )
@@ -704,7 +713,8 @@ public:
       {
           while ( first != last )
           {
-              if ( (*first)->posCount() <= count_thr )
+              if ( ! (*first)->isGhost()
+                   && (*first)->posCount() <= count_thr )
               {
                   return *first;
               }
@@ -812,7 +822,7 @@ public:
     // tactical info
 
     /*!
-      \brief get estimated offside line x coodinate
+      \brief get estimated offside line x coordinate
       \return real value
     */
     const
@@ -831,7 +841,7 @@ public:
       }
 
     /*!
-      \brief get estimated defside line(offside line for opponent) x coodinate
+      \brief get estimated defside line(offside line for opponent) x coordinate
       \return real value
     */
     const
@@ -883,9 +893,9 @@ public:
     PlayerType * teammatePlayerType( const int unum ) const;
 
     /*!
-      \brief get player type Id of teammate
+      \brief get player type Id of opponent
       \param unum target teammate uniform number
-      \return player type Id. if unum is illegal, Default Id is returned.
+      \return player type Id. if unum is illegal, Unknown is returned.
     */
     const
     HeteroID opponentHeteroID( const int unum ) const
@@ -942,47 +952,6 @@ public:
                        int * sum_count,
                        int * ave_count ) const;
 
-    /*!
-      \brief get uniform number of heared pass receiver
-      \return uniform number
-    */
-    int heardReceiverNumber() const
-      {
-          return M_heard_receiver_number;
-      }
-
-    /*!
-      \brief get receive position of heared pass
-      \return const reference to heared position object
-    */
-    const
-    Vector2D & heardReceivePos() const
-      {
-          return M_heard_receive_pos;
-      }
-
-    /*!
-      \brief get time hat pass info is arrived by heare message
-      \return const reference to game time
-    */
-    const
-    GameTime & heardPassTime() const
-      {
-          return M_heard_pass_time;
-      }
-
-    const
-    GameTime & heardWaitRequestTime() const
-      {
-          return M_heard_wait_request_time;
-      }
-
-    const
-    GameTime & heardAttackRequestTime() const
-      {
-          return M_heard_attack_request_time;
-      }
-
     // some special interface
 
     /*!
