@@ -130,7 +130,7 @@ public:
       \return registry status.
     */
     virtual
-    bool handleMsgInfo( short board,
+    bool handleMsgInfo( Int16 board,
                         const std::string & msg )
       {
           return M_holder.addMsgInfo( board, msg );
