@@ -30,13 +30,15 @@
 /////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include "types.h"
-#include "handler.h"
 #include "parser_v1.h"
 
+#include "handler.h"
+#include "types.h"
+#include <iostream>
+
 namespace rcsc {
 namespace rcg {
 
@@ -67,6 +69,11 @@ ParserV1::parse( std::istream & is,
         is.read( reinterpret_cast< char* >( &info ), sizeof( dispinfo_t ) );
         if ( is.gcount() != sizeof( dispinfo_t ) )
         {
+            if ( is.eof() )
+            {
+                break;
+            }
+            //std::cerr << "failed to read dispinfo " << std::endl;
             return false;
         }
 
@@ -79,7 +86,7 @@ ParserV1::parse( std::istream & is,
 
     if ( is.eof() )
     {
-        return true;
+        return handler.handleEOF();
     }
 
     return false;
