@@ -455,6 +455,14 @@ TrainerAgent::analyzeInit( const char * msg )
         doEar( true );
     }
 
+    // set compression level
+    if ( 0 < config().compression()
+         && config().compression() <= 9 )
+    {
+        TrainerCompressionCommand com( config().compression() );
+        sendCommand( com );
+    }
+
     M_client->setIntervalMSec( ServerParam::i().simulatorStep() );
 }
 
@@ -518,7 +526,7 @@ TrainerAgent::analyzeSeeGlobal( const char * msg )
 void
 TrainerAgent::analyzeHear( const char * msg )
 {
-    std::cerr << "Trainer hear " << msg << std::endl;
+    //std::cerr << "Trainer hear " << msg << std::endl;
     if ( ! analyzeCycle( msg, false ) )
     {
         return;
@@ -618,32 +626,34 @@ TrainerAgent::analyzeChangePlayerType( const char * msg )
     // "(ok change_player_type <teamname> <unum> <type>)"
 
     char teamname[32];
-    int unum = -1, type = -1;
+    int unum = -1, type_num = -1;
 
     std::memset( teamname, 0, 32 );
 
     if ( std::sscanf( msg, "(ok change_player_type %31[^ ] %d %d)",
-                      teamname, &unum, &type ) == 3 )
+                      teamname, &unum, &type_num ) == 3 )
     {
+        HeteroID id = static_cast< HeteroID >( type_num );
+
         if ( world().teamNameLeft() == teamname )
         {
-            M_worldmodel.setPlayerType( LEFT, unum, static_cast< HeteroID >( type ) );
+            M_worldmodel.recvChangePlayerType( LEFT, unum, id );
         }
         else if ( world().teamNameRight() == teamname )
         {
-            M_worldmodel.setPlayerType( RIGHT, unum, static_cast< HeteroID >( type ) );
+            M_worldmodel.recvChangePlayerType( RIGHT, unum, id );
         }
         else if ( world().teamNameLeft().empty()
                   && std::strlen( teamname ) != 0 )
         {
             M_worldmodel.setTeamName( LEFT, teamname );
-            M_worldmodel.setPlayerType( LEFT, unum, static_cast< HeteroID >( type ) );
+            M_worldmodel.recvChangePlayerType( LEFT, unum, id );
         }
         else if ( world().teamNameRight().empty()
                   && std::strlen( teamname ) != 0 )
         {
             M_worldmodel.setTeamName( RIGHT, teamname );
-            M_worldmodel.setPlayerType( RIGHT, unum, static_cast< HeteroID >( type ) );
+            M_worldmodel.recvChangePlayerType( RIGHT, unum, id );
         }
     }
 
