@@ -51,13 +51,21 @@ private:
     //! key: playmode string, value: playmode type
     PlayModeMap M_playmode_map;
 
-public:
-
     /*!
       \brief private access for singleton. create playmode map.
     */
     MapHolder();
 
+public:
+
+    static
+    const
+    MapHolder & instance()
+      {
+          static MapHolder s_instance;
+          return s_instance;
+      }
+
     /*!
       \brief singleton interface. get playmode map
       \return const reference to the playmode map
@@ -197,6 +205,7 @@ GameMode::update( const std::string & mode_str,
     // we should not "immediately" change playmode to know what this is goalie kick phase.
     if ( M_type == GoalieCatch_ // last playmode is goalie_catch
          && mode_pair.first == FreeKick_ // new playmode is freekick
+         && M_side == mode_pair.second // same side
          && M_time == current )   // and alse, same cycle
     {
         // do nothing
@@ -220,10 +229,10 @@ GameMode::update( const std::string & mode_str,
 GameMode::Pair
 GameMode::parse( const std::string & mode_str )
 {
-    static const MapHolder s_map_holder;
+    const PlayModeMap & pmap = MapHolder::instance().get();
 
-    PlayModeMap::const_iterator it = s_map_holder.get().find( mode_str );
-    if ( it != s_map_holder.get().end() )
+    PlayModeMap::const_iterator it = pmap.find( mode_str );
+    if ( it != pmap.end() )
     {
         return it->second;
     }
