@@ -192,17 +192,40 @@ TextPrinter::TextPrinter( std::ostream & os )
 bool
 TextPrinter::handleDispInfo( const rcsc::rcg::dispinfo_t & disp )
 {
+    //std::cerr << "handleDispInfo size = " << sizeof( disp ) << std::endl;
     switch ( ntohs( disp.mode ) ) {
     case rcsc::rcg::SHOW_MODE:
+        //std::cerr << "SHOW_MODE" << std::endl;
         handleShowInfo( disp.body.show );
         break;
     case rcsc::rcg::MSG_MODE:
+        //td::cerr << "MSG_MODE" << std::endl;
         break;
     case rcsc::rcg::DRAW_MODE:
+        //std::cerr << "DRAW_MODE" << std::endl;
+        break;
+    case rcsc::rcg::BLANK_MODE:
+        //std::cerr << "BLANK_MODE" << std::endl;
+        break;
+    case rcsc::rcg::PM_MODE:
+        //std::cerr << "PM_MODE" << std::endl;
+        break;
+    case rcsc::rcg::TEAM_MODE:
+        //std::cerr << "TEAM_MODE" << std::endl;
+        break;
+    case rcsc::rcg::PT_MODE:
+        //std::cerr << "PT_MODE" << std::endl;
+        break;
+    case rcsc::rcg::PARAM_MODE:
+        //std::cerr << "PARAM_MODE" << std::endl;
+        break;
+    case rcsc::rcg::PPARAM_MODE:
+        //std::cerr << "PPARAM_MODE" << std::endl;
         break;
     default:
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " Unsupported mode in dispinfo_t" << std::endl;
+                  << " Unsupported mode " << ntohs( disp.mode )
+                  << " in dispinfo_t" << std::endl;
         return false;
         break;
     }
@@ -352,6 +375,7 @@ TextPrinter::handleTeamInfo( const rcsc::rcg::team_t& team_left,
 bool
 TextPrinter::handlePlayerType( const rcsc::rcg::player_type_t & param )
 {
+    std::cerr << "handlePlayerType" << std::endl;
     return true;
 }
 
@@ -392,6 +416,7 @@ TextPrinter::handleServerParam( const rcsc::rcg::server_params_t & param )
 bool
 TextPrinter::handlePlayerParam( const rcsc::rcg::player_params_t & param )
 {
+    std::cerr << "handlePlayerParam" << std::endl;
     return true;
 }
 
